@@ -21,28 +21,46 @@
 
 */
 
-#include <config.h>
-#include <signal.h>		/* kill() */
+/** \file
+ *  \brief Source: editor file locking
+ *  \author Adam Byrtek
+ *  \date 2003
+ *
+ *  Locking scheme used in mcedit is based on a documentation found
+ *  in JED editor sources. Abstract from lock.c file (by John E. Davis):
+ *
+ *  The basic idea here is quite simple.  Whenever a buffer is attached to
+ *  a file, and that buffer is modified, then attempt to lock the
+ *  file. Moreover, before writing to a file for any reason, lock the
+ *  file. The lock is really a protocol respected and not a real lock.
+ *  The protocol is this: If in the directory of the file is a
+ *  symbolic link with name ".#FILE", the FILE is considered to be locked
+ *  by the process specified by the link.
+ */
 
+#include <config.h>
+
+#include <signal.h>		/* kill() */
 #include <stdio.h>
 #include <stdarg.h>
 #include <sys/types.h>
-#ifdef HAVE_UNISTD_H
-#    include <unistd.h>
-#endif
+#include <unistd.h>
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
 #include <sys/stat.h>
-
+#include <pwd.h>
 #include <stdlib.h>
 
 #include "../src/global.h"
 
-#include "edit.h"
+#include "edit-impl.h"
 #include "editlock.h"
 
 #include "../src/wtools.h"	/* edit_query_dialog () */
+#include "../src/strutil.h"	/* utf string functions */
+
+#include "../vfs/vfs.h"
 
 #define BUF_SIZE 255
 #define PID_BUF_SIZE 10
@@ -52,19 +70,10 @@ struct lock_s {
     pid_t pid;
 };
 
-/* Locking scheme used in mcedit is based on a documentation found
-   in JED editor sources. Abstract from lock.c file (by John E. Davis):
-
-   The basic idea here is quite simple.  Whenever a buffer is attached to
-   a file, and that buffer is modified, then attempt to lock the
-   file. Moreover, before writing to a file for any reason, lock the
-   file. The lock is really a protocol respected and not a real lock.
-   The protocol is this: If in the directory of the file is a
-   symbolic link with name ".#FILE", the FILE is considered to be locked
-   by the process specified by the link.
-*/
-
-/* Build user@host.domain.pid string (need to be freed) */
+/** \fn static char * lock_build_name (void)
+ *  \brief builds user@host.domain.pid string (need to be freed)
+ *  \return a pointer to lock filename
+ */
 static char *
 lock_build_name (void)
 {
@@ -79,7 +88,7 @@ lock_build_name (void)
     if (!user) user = getenv ("LOGNAME");
     if (!user) user = "";
 
-    /* TODO: Use FQDN, no clean interface, so requires lot of code */
+    /** \todo Use FQDN, no clean interface, so requires lot of code */
     if (gethostname (host, BUF_SIZE - 1) == -1)
 	*host = '\0';
 
@@ -113,7 +122,7 @@ lock_extract_info (const char *str)
     static char pid[PID_BUF_SIZE], who[BUF_SIZE];
     static struct lock_s lock;
 
-    for (p = str + strlen (str) - 1; p >= str; p--)
+    for (p = str + str_term_width1 (str) - 1; p >= str; p--)
 	if (*p == '.')
 	    break;
 
@@ -126,7 +135,7 @@ lock_extract_info (const char *str)
     /* Treat text between '.' and ':' or '\0' as pid */
     i = 0;
     for (p = p + 1;
-	 p < str + strlen (str) && *p != ':' && i < PID_BUF_SIZE; p++)
+	 p < str + str_term_width1 (str) && *p != ':' && i < PID_BUF_SIZE; p++)
 	pid[i++] = *p;
     pid[i] = '\0';
 
