@@ -18,9 +18,30 @@
    02110-1301, USA.
  */
 
+/** \file
+ *  \brief Source: word-processor mode for the editor: does dynamic paragraph formatting
+ *  \author Paul Sheer
+ *  \date 1996
+ */
+
 #include <config.h>
-#include "edit.h"
+
+#include <stdio.h>
+#include <stdarg.h>
+#include <sys/types.h>
+#include <unistd.h>
+#include <string.h>
+#include <ctype.h>
+#include <errno.h>
+#include <sys/stat.h>
+
+#include <stdlib.h>
+
+#include "../src/global.h"
+
+#include "edit-impl.h"
 #include "edit-widget.h"
+#include "../src/main.h"	/* option_tab_spacing */
 
 #define tab_width option_tab_spacing
 
@@ -74,7 +95,7 @@ static int bad_line_start (WEdit * edit, long p)
 static long
 begin_paragraph (WEdit *edit, int force)
 {
-    int i;
+    long i;
     for (i = edit->curs_line - 1; i >= 0; i--) {
 	if (line_is_blank (edit, i)) {
 	    i++;
@@ -120,13 +141,13 @@ get_paragraph (WEdit *edit, long p, long q, int indent, int *size)
 {
     unsigned char *s, *t;
 #if 0
-    t = g_malloc ((q - p) + 2 * (q - p) / option_word_wrap_line_length +
+    t = g_try_malloc ((q - p) + 2 * (q - p) / option_word_wrap_line_length +
 		  10);
 #else
-    t = g_malloc (2 * (q - p) + 100);
+    t = g_try_malloc (2 * (q - p) + 100);
 #endif
-    if (!t)
-	return 0;
+    if (t == NULL)
+	return NULL;
     for (s = t; p < q; p++, s++) {
 	if (indent)
 	    if (edit_get_byte (edit, p - 1) == '\n')
@@ -139,26 +160,28 @@ get_paragraph (WEdit *edit, long p, long q, int indent, int *size)
     return t;
 }
 
-static void strip_newlines (unsigned char *t, int size)
+static inline void
+strip_newlines (unsigned char *t, int size)
 {
     unsigned char *p = t;
-    while (size--) {
-	*p = *p == '\n' ? ' ' : *p;
+    while (size-- != 0) {
+	if (*p == '\n')
+	    *p = ' ';
 	p++;
     }
 }
 
-/* 
-   This is a copy of the function 
-   int calc_text_pos (WEdit * edit, long b, long *q, int l)
-   in propfont.c  :(
-   It calculates the number of chars in a line specified to length l in pixels
+/*
+   This function calculates the number of chars in a line specified to length l in pixels
  */
-static inline int next_tab_pos (int x)
+static inline int
+next_tab_pos (int x)
 {
     return x += tab_width - x % tab_width;
 }
-static int line_pixel_length (unsigned char *t, long b, int l)
+
+static inline int
+line_pixel_length (unsigned char *t, long b, int l)
 {
     int x = 0, c, xn = 0;
     for (;;) {
@@ -181,29 +204,32 @@ static int line_pixel_length (unsigned char *t, long b, int l)
     return b;
 }
 
-/* find the start of a word */
-static int next_word_start (unsigned char *t, int q, int size)
+static int
+next_word_start (unsigned char *t, int q, int size)
 {
     int i;
-    for (i = q;; i++) {
+    int saw_ws = 0;
+
+    for (i = q; i < size; i++) {
 	switch (t[i]) {
 	case '\n':
 	    return -1;
 	case '\t':
 	case ' ':
-	    for (;; i++) {
-		if (t[i] == '\n')
-		    return -1;
-		if (t[i] != ' ' && t[i] != '\t')
-		    return i;
-	    }
+	    saw_ws = 1;
+	    break;
+	default:
+	    if (saw_ws != 0)
+		return i;
 	    break;
 	}
     }
+    return -1;
 }
 
 /* find the start of a word */
-static int word_start (unsigned char *t, int q, int size)
+static inline int
+word_start (unsigned char *t, int q, int size)
 {
     int i = q;
     if (t[q] == ' ' || t[q] == '\t')
@@ -222,7 +248,8 @@ static int word_start (unsigned char *t, int q, int size)
 }
 
 /* replaces ' ' with '\n' to properly format a paragraph */
-static void format_this (unsigned char *t, int size, int indent)
+static inline void
+format_this (unsigned char *t, int size, int indent)
 {
     int q = 0, ww;
     strip_newlines (t, size);
@@ -238,8 +265,8 @@ static void format_this (unsigned char *t, int size, int indent)
 	    break;
 	p = word_start (t, q, size);
 	if (p == -1)
-	    q = next_word_start (t, q, size);	/* Return the end of the word if the beginning 
-						   of the word is at the beginning of a line 
+	    q = next_word_start (t, q, size);	/* Return the end of the word if the beginning
+						   of the word is at the beginning of a line
 						   (i.e. a very long word) */
 	else
 	    q = p;
@@ -250,15 +277,17 @@ static void format_this (unsigned char *t, int size, int indent)
     }
 }
 
-static void replace_at (WEdit * edit, long q, int c)
+static inline void
+replace_at (WEdit * edit, long q, int c)
 {
     edit_cursor_move (edit, q - edit->curs1);
-    edit_delete (edit);
+    edit_delete (edit, 1);
     edit_insert_ahead (edit, c);
 }
 
 /* replaces a block of text */
-static void put_paragraph (WEdit * edit, unsigned char *t, long p, long q, int indent, int size)
+static inline void
+put_paragraph (WEdit * edit, unsigned char *t, long p, int indent, int size)
 {
     long cursor;
     int i, c = 0;
@@ -282,7 +311,7 @@ static void put_paragraph (WEdit * edit, unsigned char *t, long p, long q, int i
 	    } else if (c == '\n') {
 		edit_cursor_move (edit, p - edit->curs1);
 		while (strchr ("\t ", edit_get_byte (edit, p))) {
-		    edit_delete (edit);
+		    edit_delete (edit, 1);
 		    if (cursor > edit->curs1)
 			cursor--;
 		}
@@ -296,7 +325,8 @@ static void put_paragraph (WEdit * edit, unsigned char *t, long p, long q, int i
     edit_cursor_move (edit, cursor - edit->curs1);	/* restore cursor position */
 }
 
-static int test_indent (WEdit * edit, long p, long q)
+static inline int
+test_indent (WEdit * edit, long p, long q)
 {
     int indent;
     indent = edit_indent_width (edit, p++);
@@ -342,7 +372,7 @@ format_paragraph (WEdit *edit, int force)
 	}
     }
     format_this (t, q - p, indent);
-    put_paragraph (edit, t, p, q, indent, size);
+    put_paragraph (edit, t, p, indent, size);
     g_free (t);
 
     /* Scroll left as much as possible to show the formatted paragraph */
