@@ -1,5 +1,6 @@
 /* Chown-advanced command -- for the Midnight Commander
-   Copyright (C) 1994, 1995 Radek Doulik
+   Copyright (C) 1994, 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
+   2005, 2007 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,30 +17,38 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
+/** \file achown.c
+ *  \brief Source: Contains functions for advanced chowning
+ */
+
 #include <config.h>
-/* Needed for the extern declarations of integer parameters */
+
+#include <errno.h>
+#include <stdio.h>
+#include <string.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
-#endif
-#include <string.h>
-#include <stdio.h>
-#include <errno.h>	/* For errno on SunOS systems	      */
+#include <unistd.h>
+#include <pwd.h>
+#include <grp.h>
 
 #include "global.h"
-#include "tty.h"
-#include "win.h"
-#include "color.h"
+
+#include "../src/tty/tty.h"
+#include "../src/tty/key.h"		/* XCTRL and ALT macros */
+#include "../src/skin/skin.h"
+
 #include "dialog.h"
 #include "widget.h"
 #include "wtools.h"		/* For init_box_colors() */
-#include "key.h"		/* XCTRL and ALT macros */
+#include "strutil.h"
 
 #include "dir.h"
 #include "panel.h"		/* Needed for the externs */
 #include "chmod.h"
-#include "main.h"
+#include "main.h"		/* update_panels() */
+#include "layout.h"		/* repaint_screen() */
 #include "achown.h"
 
 #define BX		5
@@ -68,12 +77,12 @@ static struct {
     { B_CANCEL, NORMAL_BUTTON, 4, 53, N_("&Cancel") },
     { B_ENTER,  DEFPUSH_BUTTON,4, 40, N_("&Set") },
     { B_SKIP,   NORMAL_BUTTON, 4, 23, N_("S&kip") },
-    { B_SETALL, NORMAL_BUTTON, 4, 0, N_("Set &all")},
-    { B_ENTER,  NARROW_BUTTON, 0, 47, "               "},
-    { B_ENTER,  NARROW_BUTTON, 0, 29, "               "},
+    { B_SETALL, NORMAL_BUTTON, 4,  0, N_("Set &all")},
+    { B_ENTER,  NARROW_BUTTON, 0, 47, ""},
+    { B_ENTER,  NARROW_BUTTON, 0, 29, ""},
     { B_ENTER,  NARROW_BUTTON, 0, 19, "   "},
     { B_ENTER,  NARROW_BUTTON, 0, 11, "   "},
-    { B_ENTER,  NARROW_BUTTON, 0, 3, "   "}
+    { B_ENTER,  NARROW_BUTTON, 0,  3, "   "}
 };
 
 static WButton *b_att[3];	/* permission */
@@ -84,7 +93,7 @@ static int flag_pos;
 static int x_toggle;
 static char ch_flags[11];
 static const char ch_perm[] = "rwx";
-static umode_t ch_cmode;
+static mode_t ch_cmode;
 static struct stat *sf_stat;
 static int need_update;
 static int end_chown;
@@ -92,20 +101,14 @@ static int current_file;
 static int single_set;
 static char *fname;
 
-static void get_ownership (void)
-{				/* set buttons  - ownership */
-    const char *name_t;
-
-    name_t = name_trunc (get_owner (sf_stat->st_uid), 15);
-    memset (b_user->text, ' ', 15);
-    strncpy (b_user->text, name_t, strlen (name_t));
-    name_t = name_trunc (get_group (sf_stat->st_gid), 15);
-    memset (b_group->text, ' ', 15);
-    strncpy (b_group->text, name_t, strlen (name_t));
+static void update_ownership (void)
+{
+    button_set_text (b_user, get_owner (sf_stat->st_uid));
+    button_set_text (b_group, get_group (sf_stat->st_gid));
 }
 
 
-static int inc_flag_pos (int f_pos)
+static cb_ret_t inc_flag_pos (int f_pos)
 {
     if (flag_pos == 10) {
 	flag_pos = 0;
@@ -143,26 +146,33 @@ static void set_perm_by_flags (char *s, int f_p)
     }
 }
 
-static umode_t get_perm (char *s, int base)
+static void update_permissions (void)
 {
-    umode_t m;
+    set_perm_by_flags (b_att[0]->text.start, 0);
+    set_perm_by_flags (b_att[1]->text.start, 3);
+    set_perm_by_flags (b_att[2]->text.start, 6);
+}
+
+static mode_t get_perm (char *s, int base)
+{
+    mode_t m;
 
     m = 0;
-    m |= (s [0] == '-') ? 0 :
-	((s[0] == '+') ? (1 << (base + 2)) : (1 << (base + 2)) & ch_cmode);
+    m |= (s[0] == '-') ? 0 :
+	((s[0] == '+') ? (mode_t)(1 << (base + 2)) : (1 << (base + 2)) & ch_cmode);
 
-    m |= (s [1] == '-') ? 0 :
-	((s[1] == '+') ? (1 << (base + 1)) : (1 << (base + 1)) & ch_cmode);
+    m |= (s[1] == '-') ? 0 :
+	((s[1] == '+') ? (mode_t)(1 << (base + 1)) : (1 << (base + 1)) & ch_cmode);
     
-    m |= (s [2] == '-') ? 0 :
-	((s[2] == '+') ? (1 << base) : (1 << base) & ch_cmode);
+    m |= (s[2] == '-') ? 0 :
+	((s[2] == '+') ? (mode_t)(1 << base) : (1 << base) & ch_cmode);
 
     return m;
 }
 
-static umode_t get_mode (void)
+static mode_t get_mode (void)
 {
-    umode_t m;
+    mode_t m;
 
     m = ch_cmode ^ (ch_cmode & 0777);
     m |= get_perm (ch_flags, 6);
@@ -176,48 +186,47 @@ static void print_flags (void)
 {
     int i;
 
-    attrset (COLOR_NORMAL);
+    tty_setcolor (COLOR_NORMAL);
 
     for (i = 0; i < 3; i++){
 	dlg_move (ch_dlg, BY+1, 9+i);
-	addch (ch_flags [i]);
+	tty_print_char (ch_flags [i]);
     }
     
     for (i = 0; i < 3; i++){
 	dlg_move (ch_dlg, BY + 1, 17 + i);
-	addch (ch_flags [i+3]);
+	tty_print_char (ch_flags [i+3]);
     }
     
     for (i = 0; i < 3; i++){
 	dlg_move (ch_dlg, BY + 1, 25 + i);
-	addch (ch_flags [i+6]);
+	tty_print_char (ch_flags [i+6]);
     }
 
-    set_perm_by_flags (b_att[0]->text, 0);
-    set_perm_by_flags (b_att[1]->text, 3);
-    set_perm_by_flags (b_att[2]->text, 6);
+    update_permissions ();
 
     for (i = 0; i < 15; i++){
 	dlg_move (ch_dlg, BY+1, 35+i);
-	addch (ch_flags[9]);
+	tty_print_char (ch_flags[9]);
     }
     for (i = 0; i < 15; i++){
 	dlg_move (ch_dlg, BY + 1, 53 + i);
-	addch (ch_flags[10]);
+	tty_print_char (ch_flags[10]);
     }
 }
 
 static void update_mode (Dlg_head * h)
 {
     print_flags ();
-    attrset (COLOR_NORMAL);
+    tty_setcolor (COLOR_NORMAL);
     dlg_move (h, BY + 2, 9);
-    printw ("%12o", get_mode ());
+    tty_printf ("%12o", get_mode ());
     send_message (h->current, WIDGET_FOCUS, 0);
 }
 
 static cb_ret_t
-chl_callback (Dlg_head *h, dlg_msg_t msg, int parm)
+chl_callback (Dlg_head *h, Widget *sender,
+		dlg_msg_t msg, int parm, void *data)
 {
     switch (msg) {
     case DLG_KEY:
@@ -229,7 +238,7 @@ chl_callback (Dlg_head *h, dlg_msg_t msg, int parm)
 	}
 
     default:
-	return default_dlg_callback (h, msg, parm);
+	return default_dlg_callback (h, sender, msg, parm, data);
     }
 }
 
@@ -258,15 +267,18 @@ do_enter_key (Dlg_head * h, int f_pos)
 			"[Advanced Chown]", title, DLG_COMPACT | DLG_REVERSE);
 
 	/* get new listboxes */
-	chl_list = listbox_new (1, 1, 15, 11, NULL);
+	chl_list = listbox_new (1, 1, 11, 15, NULL);
 
-	listbox_add_item (chl_list, 0, 0, "<Unknown>", NULL);
+	listbox_add_item (chl_list, LISTBOX_APPEND_AT_END, 0,
+	    "<Unknown>", NULL);
 
 	if (is_owner) {
 	    /* get and put user names in the listbox */
 	    setpwent ();
-	    while ((chl_pass = getpwent ()))
-		listbox_add_item (chl_list, 0, 0, chl_pass->pw_name, NULL);
+	    while ((chl_pass = getpwent ())) {
+		listbox_add_item (chl_list, LISTBOX_APPEND_SORTED, 0,
+		    chl_pass->pw_name, NULL);
+	    }
 	    endpwent ();
 	    fe = listbox_search_text (chl_list,
 				      get_owner (sf_stat->st_uid));
@@ -274,7 +286,8 @@ do_enter_key (Dlg_head * h, int f_pos)
 	    /* get and put group names in the listbox */
 	    setgrent ();
 	    while ((chl_grp = getgrent ())) {
-		listbox_add_item (chl_list, 0, 0, chl_grp->gr_name, NULL);
+		listbox_add_item (chl_list, LISTBOX_APPEND_SORTED, 0,
+		    chl_grp->gr_name, NULL);
 	    }
 	    endgrent ();
 	    fe = listbox_search_text (chl_list,
@@ -306,7 +319,7 @@ do_enter_key (Dlg_head * h, int f_pos)
 	    }
 	    if (ok) {
 		ch_flags[f_pos + 6] = '+';
-		get_ownership ();
+		update_ownership ();
 	    }
 	    dlg_focus (h);
 	    if (ok)
@@ -332,31 +345,32 @@ static void chown_refresh (void)
 {
     common_dialog_repaint (ch_dlg);
 
-    attrset (COLOR_NORMAL);
+    tty_setcolor (COLOR_NORMAL);
 
     dlg_move (ch_dlg, BY - 1, 8);
-    addstr (_("owner"));
+    tty_print_string (_("owner"));
     dlg_move (ch_dlg, BY - 1, 16);
-    addstr (_("group"));
+    tty_print_string (_("group"));
     dlg_move (ch_dlg, BY - 1, 24);
-    addstr (_("other"));
+    tty_print_string (_("other"));
     
     dlg_move (ch_dlg, BY - 1, 35);
-    addstr (_("owner"));
+    tty_print_string (_("owner"));
     dlg_move (ch_dlg, BY - 1, 53);
-    addstr (_("group"));
+    tty_print_string (_("group"));
     
     dlg_move (ch_dlg, 3, 4);
-    addstr (_("On"));
+    tty_print_string (_("On"));
     dlg_move (ch_dlg, BY + 1, 4);
-    addstr (_("Flag"));
+    tty_print_string (_("Flag"));
     dlg_move (ch_dlg, BY + 2, 4);
-    addstr (_("Mode"));
+    tty_print_string (_("Mode"));
 
     if (!single_set){
 	dlg_move (ch_dlg, 3, 54);
-	printw (const_cast(char *, _("%6d of %d")), files_on_begin - (current_panel->marked) + 1,
-		   files_on_begin);
+	tty_printf (_("%6d of %d"),
+	    files_on_begin - (current_panel->marked) + 1,
+	    files_on_begin);
     }
 
     print_flags ();
@@ -365,18 +379,16 @@ static void chown_refresh (void)
 static void chown_info_update (void)
 {
     /* display file info */
-    attrset (COLOR_NORMAL);
+    tty_setcolor (COLOR_NORMAL);
     
     /* name && mode */
     dlg_move (ch_dlg, 3, 8);
-    printw ("%s", name_trunc (fname, 45));
+    tty_print_string (str_fit_to_term (fname, 45, J_LEFT_FIT));
     dlg_move (ch_dlg, BY + 2, 9);
-    printw ("%12o", get_mode ());
+    tty_printf ("%12o", get_mode ());
     
     /* permissions */
-    set_perm_by_flags (b_att[0]->text, 0);
-    set_perm_by_flags (b_att[1]->text, 3);
-    set_perm_by_flags (b_att[2]->text, 6);
+    update_permissions ();
 }
 
 static void b_setpos (int f_pos) {
@@ -387,7 +399,8 @@ static void b_setpos (int f_pos) {
 }
 
 static cb_ret_t
-advanced_chown_callback (Dlg_head *h, dlg_msg_t msg, int parm)
+advanced_chown_callback (Dlg_head *h, Widget *sender,
+			    dlg_msg_t msg, int parm, void *data)
 {
     int i = 0, f_pos = BUTTONS - h->current->dlg_id - single_set - 1;
 
@@ -482,7 +495,7 @@ advanced_chown_callback (Dlg_head *h, dlg_msg_t msg, int parm)
 	    if (f_pos > 2)
 		break;
 	    flag_pos = f_pos * 3 + i;	/* (strchr(ch_perm,parm)-ch_perm); */
-	    if (((WButton *) h->current)->text[(flag_pos % 3)] ==
+	    if (((WButton *) h->current)->text.start[(flag_pos % 3)] ==
 		'-')
 		ch_flags[flag_pos] = '+';
 	    else
@@ -514,20 +527,19 @@ advanced_chown_callback (Dlg_head *h, dlg_msg_t msg, int parm)
 
 	case '=':
 	case '+':
-	    if (f_pos > 4)
-		break;
-	    ch_flags[flag_pos] = parm;
-	    update_mode (h);
-	    advanced_chown_callback (h, DLG_KEY, KEY_RIGHT);
-	    if (flag_pos > 8 || !(flag_pos % 3))
-		dlg_one_down (h);
-
+	    if (f_pos <= 4) {
+		ch_flags[flag_pos] = parm;
+		update_mode (h);
+		advanced_chown_callback (h, sender, DLG_KEY, KEY_RIGHT, NULL);
+		if (flag_pos > 8 || !(flag_pos % 3))
+		    dlg_one_down (h);
+	    }
 	    break;
 	}
 	return MSG_NOT_HANDLED;
 
     default:
-	return default_dlg_callback (h, msg, parm);
+	return default_dlg_callback (h, sender, msg, parm, data);
     }
 }
 
@@ -537,13 +549,13 @@ init_chown_advanced (void)
     int i;
     enum { dlg_h = 13, dlg_w = 74, n_elem = 4 };
 #ifdef ENABLE_NLS
-    static int i18n_len;
-    
-    if (!i18n_len) {
+    static int i18n_len = 0;
+
+    if (i18n_len == 0) {
 	int dx, cx;
 	for (i = 0 ; i < n_elem ; i++) {
 	    chown_advanced_but[i].text = _(chown_advanced_but[i].text);
-	    i18n_len += strlen (chown_advanced_but[i].text) + 3;
+	    i18n_len += str_term_width1 (chown_advanced_but[i].text) + 3;
 	    if (DEFPUSH_BUTTON == chown_advanced_but[i].flags)
 		i18n_len += 2; /* "<>" */ 
 	}
@@ -552,7 +564,7 @@ init_chown_advanced (void)
 	/* Reversed order */
 	for (i = n_elem - 1; i >= 0; i--) {
 	    chown_advanced_but[i].x = cx;
-	    cx += strlen (chown_advanced_but[i].text) + 3 + dx;
+	    cx += str_term_width1 (chown_advanced_but[i].text) + 3 + dx;
 	}
     }
 #endif /* ENABLE_NLS */
@@ -601,7 +613,7 @@ chown_advanced_done (void)
 }
 
 #if 0
-static inline void do_chown (uid_t u, gid_t g)
+static void do_chown (uid_t u, gid_t g)
 {
     chown (current_panel->dir.list[current_file].fname, u, g);
     file_mark (current_panel, current_file, 0);
@@ -618,36 +630,36 @@ static char *next_file (void)
 
 static void apply_advanced_chowns (struct stat *sf)
 {
-    char *fname;
+    char *lc_fname;
     gid_t a_gid = sf->st_gid;
     uid_t a_uid = sf->st_uid;
 
-    fname = current_panel->dir.list[current_file].fname;
+    lc_fname = current_panel->dir.list[current_file].fname;
     need_update = end_chown = 1;
-    if (mc_chmod (fname, get_mode ()) == -1)
-	message (1, MSG_ERROR, _(" Cannot chmod \"%s\" \n %s "),
-		 fname, unix_error_string (errno));
+    if (mc_chmod (lc_fname, get_mode ()) == -1)
+	message (D_ERROR, MSG_ERROR, _(" Cannot chmod \"%s\" \n %s "),
+		 lc_fname, unix_error_string (errno));
     /* call mc_chown only, if mc_chmod didn't fail */
-    else if (mc_chown (fname, (ch_flags[9] == '+') ? sf->st_uid : (uid_t) -1,
+    else if (mc_chown (lc_fname, (ch_flags[9] == '+') ? sf->st_uid : (uid_t) -1,
 		       (ch_flags[10] == '+') ? sf->st_gid : (gid_t) -1) == -1)
-	message (1, MSG_ERROR, _(" Cannot chown \"%s\" \n %s "),
-		 fname, unix_error_string (errno));
+	message (D_ERROR, MSG_ERROR, _(" Cannot chown \"%s\" \n %s "),
+		 lc_fname, unix_error_string (errno));
     do_file_mark (current_panel, current_file, 0);
 
     do {
-	fname = next_file ();
+	lc_fname = next_file ();
 
-	if (mc_stat (fname, sf) != 0)
+	if (mc_stat (lc_fname, sf) != 0)
 	    break;
 	ch_cmode = sf->st_mode;
-	if (mc_chmod (fname, get_mode ()) == -1)
-	    message (1, MSG_ERROR, _(" Cannot chmod \"%s\" \n %s "),
-		     fname, unix_error_string (errno));
+	if (mc_chmod (lc_fname, get_mode ()) == -1)
+	    message (D_ERROR, MSG_ERROR, _(" Cannot chmod \"%s\" \n %s "),
+		     lc_fname, unix_error_string (errno));
 	/* call mc_chown only, if mc_chmod didn't fail */
-	else if (mc_chown (fname, (ch_flags[9] == '+') ? a_uid : (uid_t) -1,
+	else if (mc_chown (lc_fname, (ch_flags[9] == '+') ? a_uid : (uid_t) -1,
 	                   (ch_flags[10] == '+') ? a_gid : (gid_t) -1) == -1)
-	    message (1, MSG_ERROR, _(" Cannot chown \"%s\" \n %s "),
-		     fname, unix_error_string (errno));
+	    message (D_ERROR, MSG_ERROR, _(" Cannot chown \"%s\" \n %s "),
+		     lc_fname, unix_error_string (errno));
 
 	do_file_mark (current_panel, current_file, 0);
     } while (current_panel->marked);
@@ -675,7 +687,7 @@ chown_advanced_cmd (void)
 
 	chown_refresh ();
 	
-	get_ownership ();
+	update_ownership ();
 
 	/* game can begin */
 	run_dlg (ch_dlg);
@@ -688,12 +700,12 @@ chown_advanced_cmd (void)
 	case B_ENTER:
 	    need_update = 1;
 	    if (mc_chmod (fname, get_mode ()) == -1)
-		message (1, MSG_ERROR, _(" Cannot chmod \"%s\" \n %s "),
+		message (D_ERROR, MSG_ERROR, _(" Cannot chmod \"%s\" \n %s "),
 			 fname, unix_error_string (errno));
 	    /* call mc_chown only, if mc_chmod didn't fail */
 	    else if (mc_chown (fname, (ch_flags[9] == '+') ? sf_stat->st_uid : (uid_t) -1,
 	                       (ch_flags[10] == '+') ? sf_stat->st_gid : (gid_t) -1) == -1)
-		message (1, MSG_ERROR, _(" Cannot chown \"%s\" \n %s "),
+		message (D_ERROR, MSG_ERROR, _(" Cannot chown \"%s\" \n %s "),
 			 fname, unix_error_string (errno));
 	    break;
 	case B_SETALL:
