@@ -20,34 +20,38 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
+/** \file complete.c
+ *  \brief Source: Input line filename/username/hostname/variable/command completion
+ */
+
 #include <config.h>
 
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <dirent.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <pwd.h>
 #include <unistd.h>
 
-#include <mhl/memory.h>
-#include <mhl/escape.h>
-#include <mhl/string.h>
-
 #include "global.h"
-#include "tty.h"
-#include "win.h"
-#include "color.h"
+
+#include "../src/tty/tty.h"
+#include "../src/tty/key.h"		/* XCTRL and ALT macros */
+
 #include "dialog.h"
 #include "widget.h"
 #include "wtools.h"
-#include "main.h"
+#include "main.h"		/* show_all_if_ambiguous */
 #include "util.h"
-#include "key.h"		/* XCTRL and ALT macros */
+#include "../src/strescape.h"
+#include "strutil.h"
 
-typedef char *CompletionFunction (char * text, int state, INPUT_COMPLETE_FLAGS flags);
+typedef char *CompletionFunction (const char * text, int state, INPUT_COMPLETE_FLAGS flags);
 
-//#define DO_COMPLETION_DEBUG
+/* #define DO_COMPLETION_DEBUG */
 #ifdef DO_COMPLETION_DEBUG
 /*
  * Useful to print/debug completion flags
@@ -72,7 +76,7 @@ static const char * show_c_flags(INPUT_COMPLETE_FLAGS flags)
 #endif /* DO_CMPLETION_DEBUG */
 
 static char *
-filename_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
+filename_completion_function (const char * text, int state, INPUT_COMPLETE_FLAGS flags)
 {
     static DIR *directory;
     static char *filename = NULL;
@@ -86,7 +90,21 @@ filename_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
     SHOW_C_CTX("filename_completion_function");
 
     if (text && (flags & INPUT_COMPLETE_SHELL_ESC))
-        text = mhl_shell_unescape_buf (text);
+    {
+        char * u_text;
+        char * result;
+        char * e_result;
+
+        u_text = strutils_shell_unescape (text);
+
+        result = filename_completion_function (u_text, state, flags & (~INPUT_COMPLETE_SHELL_ESC));
+        g_free (u_text);
+
+        e_result = strutils_shell_escape (result);
+        g_free (result);
+
+        return e_result;
+    }
 
     /* If we're starting the match process, initialize us a bit. */
     if (!state){
@@ -108,14 +126,13 @@ filename_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
 
         /* Save the version of the directory that the user typed. */
         users_dirname = dirname;
-        {
-	    // FIXME: memleak ?
-	    dirname = tilde_expand (dirname);
-	    canonicalize_pathname (dirname);
-	    /* Here we should do something with variable expansion
-	       and `command`.
-	       Maybe a dream - UNIMPLEMENTED yet. */
-        }
+	dirname = tilde_expand (dirname);
+	canonicalize_pathname (dirname);
+
+	/* Here we should do something with variable expansion
+	   and `command`.
+	   Maybe a dream - UNIMPLEMENTED yet. */
+
         directory = mc_opendir (dirname);
         filename_len = strlen (filename);
     }
@@ -123,9 +140,12 @@ filename_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
     /* Now that we have some state, we can read the directory. */
 
     while (directory && (entry = mc_readdir (directory))){
+	if (!str_is_valid_string (entry->d_name))
+	    continue;
+
         /* Special case for no filename.
 	   All entries except "." and ".." match. */
-        if (!filename_len){
+	if (filename_len == 0) {
 	    if (!strcmp (entry->d_name, ".") || !strcmp (entry->d_name, ".."))
 	        continue;
 	} else {
@@ -138,18 +158,16 @@ filename_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
 	}
 	isdir = 1; isexec = 0;
 	{
-	    char *tmp = g_malloc (3 + strlen (dirname) + NLENGTH (entry));
+	    char *tmp;
 	    struct stat tempstat;
-	    
-	    strcpy (tmp, dirname);
-	    strcat (tmp, PATH_SEP_STR);
-	    strcat (tmp, entry->d_name);
+	
+	    tmp = g_strconcat (dirname, PATH_SEP_STR, entry->d_name, (char *) NULL);
 	    canonicalize_pathname (tmp);
 	    /* Unix version */
 	    if (!mc_stat (tmp, &tempstat)){
 	    	uid_t my_uid = getuid ();
 	    	gid_t my_gid = getgid ();
-	    	
+	
 	        if (!S_ISDIR (tempstat.st_mode)){
 	            isdir = 0;
 	            if ((!my_uid && (tempstat.st_mode & 0111)) ||
@@ -159,7 +177,12 @@ filename_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
 	                isexec = 1;
 	        }
 	    }
-	   g_free (tmp);
+	    else
+	    {
+	        /* stat failed, strange. not a dir in any case */
+	        isdir = 0;
+	    }
+	    g_free (tmp);
 	}
 	if ((flags & INPUT_COMPLETE_COMMANDS)
 	    && (isexec || isdir))
@@ -169,7 +192,6 @@ filename_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
 	    break;
 	if (flags & (INPUT_COMPLETE_FILENAMES))
 	    break;
-	continue;
     }
 
     if (!entry){
@@ -188,7 +210,7 @@ filename_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
         char *temp;
 
         if (users_dirname && (users_dirname[0] != '.' || users_dirname[1])){
-	    int dirlen = strlen (users_dirname);
+	    size_t dirlen = strlen (users_dirname);
 	    temp = g_malloc (3 + dirlen + NLENGTH (entry));
 	    strcpy (temp, users_dirname);
 	    /* We need a `/' at the end. */
@@ -204,12 +226,6 @@ filename_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
 	if (isdir)
 	    strcat (temp, PATH_SEP_STR);
 
-	if (temp && (flags & INPUT_COMPLETE_SHELL_ESC))
-	{
-	    SHELL_ESCAPED_STR e_temp = mhl_shell_escape_dup(temp);
-	    mhl_mem_free (temp);
-	    temp = e_temp.s;
-	}
 	return temp;
     }
 }
@@ -217,38 +233,34 @@ filename_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
 /* We assume here that text[0] == '~' , if you want to call it in another way,
    you have to change the code */
 static char *
-username_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
+username_completion_function (const char *text, int state, INPUT_COMPLETE_FLAGS flags)
 {
     static struct passwd *entry;
-    static int userlen;
+    static size_t userlen;
 
+    (void) flags;
     SHOW_C_CTX("username_completion_function");
 
-    if (text[0] == '\\' && text[1] == '~') text++;
+    if (text[0] == '\\' && text[1] == '~')
+	text++;
     if (!state){ /* Initialization stuff */
         setpwent ();
         userlen = strlen (text + 1);
     }
     while ((entry = getpwent ()) != NULL){
         /* Null usernames should result in all users as possible completions. */
-        if (!userlen)
+        if (userlen == 0)
             break;
-        else if (text[1] == entry->pw_name[0] &&
-	         !strncmp (text + 1, entry->pw_name, userlen))
+        if (text[1] == entry->pw_name[0]
+	    && !strncmp (text + 1, entry->pw_name, userlen))
 	    break;
     }
 
-    if (!entry){
-        endpwent ();
-        return NULL;
-    } else {
-        char *temp = g_malloc (3 + strlen (entry->pw_name));
-        
-        *temp = '~';
-        strcpy (temp + 1, entry->pw_name);
-        strcat (temp, PATH_SEP_STR);
-        return temp;
-    }
+    if (entry)
+	return g_strconcat ("~", entry->pw_name, PATH_SEP_STR, (char *) NULL);
+
+    endpwent ();
+    return NULL;
 }
 
 /* Linux declares environ in <unistd.h>, so don't repeat it here. */
@@ -259,12 +271,13 @@ extern char **environ;
 /* We assume text [0] == '$' and want to have a look at text [1], if it is
    equal to '{', so that we should append '}' at the end */
 static char *
-variable_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
+variable_completion_function (const char *text, int state, INPUT_COMPLETE_FLAGS flags)
 {
     static char **env_p;
     static int varlen, isbrace;
     const char *p = NULL;
 
+    (void) flags;
     SHOW_C_CTX("variable_completion_function");
 
     if (!state){ /* Initialization stuff */
@@ -308,20 +321,24 @@ static void fetch_hosts (const char *filename)
 {
     FILE *file = fopen (filename, "r");
     char buffer[256], *name;
-    register int i, start;
+    char *start;
+    char *bi;
 
     if (!file)
         return;
 
     while (fgets (buffer, 255, file) != NULL){
         /* Skip to first character. */
-        for (i = 0; buffer[i] && cr_whitespace (buffer[i]); i++);
+        for (bi = buffer; 
+             bi[0] != '\0' && str_isspace (bi); 
+             str_next_char (&bi));
+        
         /* Ignore comments... */
-        if (buffer[i] == '#')
+        if (bi[0] == '#')
             continue;
         /* Handle $include. */
-        if (!strncmp (buffer + i, "$include ", 9)){
-	    char *includefile = buffer + i + 9;
+        if (!strncmp (bi, "$include ", 9)){
+	    char *includefile = bi + 9;
 	    char *t;
 
 	    /* Find start of filename. */
@@ -330,8 +347,8 @@ static void fetch_hosts (const char *filename)
 	    t = includefile;
 
 	    /* Find end of filename. */
-	    while (*t && !cr_whitespace (*t))
-	        t++;
+	    while (t[0] != '\0' && !str_isspace (t))
+	        str_next_char (&t);
 	    *t = '\0';
 
 	    fetch_hosts (includefile);
@@ -339,19 +356,22 @@ static void fetch_hosts (const char *filename)
 	}
 
         /* Skip IP #s. */
-	while (buffer[i] && !cr_whitespace (buffer[i]))
-	    i++;
+	while (bi[0] != '\0' && !str_isspace (bi))
+	    str_next_char (&bi);
 
         /* Get the host names separated by white space. */
-        while (buffer[i] && buffer[i] != '#'){
-	    while (buffer[i] && cr_whitespace (buffer[i]))
-		i++;
-	    if (buffer[i] ==  '#')
+        while (bi[0] != '\0' && bi[0] != '#'){
+	    while (bi[0] != '\0' && str_isspace (bi))
+		str_next_char (&bi);
+	    if (bi[0] ==  '#')
 		continue;
-	    for (start = i; buffer[i] && !cr_whitespace (buffer[i]); i++);
-	        if (i - start == 0)
-	            continue;
-	    name = g_strndup (buffer + start, i - start);
+	    for (start = bi; 
+                 bi[0] != '\0' && !str_isspace (bi); 
+                 str_next_char (&bi));
+            
+	    if (bi - start == 0) continue;
+            
+	    name = g_strndup (start, bi - start);
 	    {
 	    	char **host_p;
 	    	
@@ -376,11 +396,12 @@ static void fetch_hosts (const char *filename)
 }
 
 static char *
-hostname_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
+hostname_completion_function (const char *text, int state, INPUT_COMPLETE_FLAGS flags)
 {
     static char **host_p;
     static int textstart, textlen;
 
+    (void) flags;
     SHOW_C_CTX("hostname_completion_function");
 
     if (!state){ /* Initialization stuff */
@@ -432,10 +453,11 @@ hostname_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
  * table of shell built-ins.
  */
 static char *
-command_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
+command_completion_function (const char *_text, int state, INPUT_COMPLETE_FLAGS flags)
 {
+    char *text;
     static const char *path_end;
-    static int isabsolute;
+    static gboolean isabsolute;
     static int phase;
     static int text_len;
     static const char *const *words;
@@ -462,12 +484,11 @@ command_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
 
     if (!(flags & INPUT_COMPLETE_COMMANDS))
         return 0;
-
-    text = mhl_shell_unescape_buf(text);
+    text = strutils_shell_unescape(_text);
     flags &= ~INPUT_COMPLETE_SHELL_ESC;
 
     if (!state) {		/* Initialize us a little bit */
-	isabsolute = strchr (text, PATH_SEP) != 0;
+	isabsolute = strchr (text, PATH_SEP) != NULL;
 	if (!isabsolute) {
 	    words = bash_reserved;
 	    phase = 0;
@@ -484,11 +505,15 @@ command_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
 
     if (isabsolute) {
 	p = filename_completion_function (text, state, flags);
-	if (!p)
-	    return 0;
-	SHELL_ESCAPED_STR e_p = mhl_shell_escape_dup(p);
-	mhl_mem_free(p);
-	return e_p.s;
+
+	if (p) {
+	    char *temp_p = p;
+	    p = strutils_shell_escape (p);
+	    g_free (temp_p);
+	}
+
+	g_free (text);
+	return p;
     }
 
     found = NULL;
@@ -520,7 +545,7 @@ command_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
 		if (cur_path >= path_end)
 		    break;
 		expanded = tilde_expand (*cur_path ? cur_path : ".");
-		cur_word = mhl_str_dir_plus_file (expanded, text);
+		cur_word = concat_dir_and_file (expanded, text);
 		g_free (expanded);
 		canonicalize_pathname (cur_word);
 		cur_path = strchr (cur_path, 0) + 1;
@@ -536,20 +561,17 @@ command_completion_function (char *text, int state, INPUT_COMPLETE_FLAGS flags)
 	}
     }
 
-    if (!found) {
+    if (found == NULL) {
 	g_free (path);
 	path = NULL;
-	return NULL;
+    } else if ((p = strrchr (found, PATH_SEP)) != NULL) {
+	char *tmp = found;
+	found = strutils_shell_escape (p + 1);
+	g_free (tmp);
     }
-    if ((p = strrchr (found, PATH_SEP)) != NULL) {
-	p++;
 
-	SHELL_ESCAPED_STR e_p = mhl_shell_escape_dup(p);
-	mhl_mem_free(found);
-	return e_p.s;
-    }
+    g_free(text);
     return found;
-
 }
 
 static int
@@ -566,7 +588,7 @@ match_compare (const void *a, const void *b)
    as the second. 
    In case no matches were found we return NULL. */
 static char **
-completion_matches (char *text, CompletionFunction entry_function, INPUT_COMPLETE_FLAGS flags)
+completion_matches (const char *text, CompletionFunction entry_function, INPUT_COMPLETE_FLAGS flags)
 {
     /* Number of slots in match_list. */
     int match_list_size;
@@ -612,19 +634,30 @@ completion_matches (char *text, CompletionFunction entry_function, INPUT_COMPLET
 	    j = i + 1;
 	    while (j < matches + 1)
 	    {
-		register int c1, c2, si;
+                char *si, *sj;
+                char *ni, *nj;
 
-		for (si = 0;(c1 = match_list [i][si]) && (c2 = match_list [j][si]); si++)
-		    if (c1 != c2) break;
+		for (si = match_list[i], sj  = match_list[j];
+                    si[0] && sj[0];) {
+
+                    ni = str_get_next_char (si);
+                    nj = str_get_next_char (sj);
 		
-		if (!c1 && !match_list [j][si]){ /* Two equal strings */
+                    if (ni - si != nj - sj) break;
+                    if (strncmp (si, sj, ni - si) != 0) break;
+
+                    si = ni;
+                    sj = nj;
+                }
+		
+                if (si[0] == '\0' && sj[0] == '\0'){ /* Two equal strings */
 		    g_free (match_list [j]);
 		    j++;
 		    if (j > matches)
 		        break;
 		    continue; /* Look for a run of equal strings */
 		} else
-	            if (low > si) low = si;
+	            if (low > si - match_list[i]) low = si - match_list[i];
 		if (i + 1 != j) /* So there's some gap */
 		    match_list [i + 1] = match_list [j];
 	        i++; j++;
@@ -644,21 +677,26 @@ completion_matches (char *text, CompletionFunction entry_function, INPUT_COMPLET
 static int
 check_is_cd (const char *text, int start, INPUT_COMPLETE_FLAGS flags)
 {
-    const char *p, *q;
+    char *p, *q;
+    int test = 0;
 
     SHOW_C_CTX("check_is_cd");
     if (!(flags & INPUT_COMPLETE_CD))
 	return 0;
 
     /* Skip initial spaces */
-    p = text;
-    q = text + start;
-    while (p < q && *p && isspace ((unsigned char) *p))
-	p++;
+    p = (char*)text;
+    q = (char*)text + start;
+    while (p < q && p[0] != '\0' && str_isspace (p))
+	str_next_char (&p);
 
     /* Check if the command is "cd" and the cursor is after it */
-    if (p[0] == 'c' && p[1] == 'd' && isspace ((unsigned char) p[2])
-	&& (p + 2 < q))
+    text+= p[0] == 'c';
+    str_next_char (&p);
+    text+= p[0] == 'd';
+    str_next_char (&p);
+    text+= str_isspace (p);
+    if (test == 3 && (p < q))
 	return 1;
 
     return 0;
@@ -669,51 +707,42 @@ static char **
 try_complete (char *text, int *start, int *end, INPUT_COMPLETE_FLAGS flags)
 {
     int in_command_position = 0;
-    char *word, c;
+    char *word;
     char **matches = NULL;
     const char *command_separator_chars = ";|&{(`";
     char *p = NULL, *q = NULL, *r = NULL;
     int is_cd = check_is_cd (text, *start, flags);
+    char *ti;
 
     SHOW_C_CTX("try_complete");
-
-    c = text [*end];
-    text [*end] = 0;
-    word = g_strdup (text + *start);
-    text [*end] = c;
+    word = g_strndup (text + *start, *end - *start);
 
     /* Determine if this could be a command word. It is if it appears at
        the start of the line (ignoring preceding whitespace), or if it
        appears after a character that separates commands. And we have to
        be in a INPUT_COMPLETE_COMMANDS flagged Input line. */
     if (!is_cd && (flags & INPUT_COMPLETE_COMMANDS)){
-        int i = *start - 1;
-	for (i = *start - 1; i > -1; i--) {
-	    if (text[i] == ' ' || text[i] == '\t'){
-		if (i == 0 ) continue;
-		if (text[i-1] != '\\') {
-		    i--;
-		    break;
-		}
-	    }
-	}
-        if (i < 0)
-	    in_command_position++;
-        else if (strchr (command_separator_chars, text[i])){
+        ti = str_get_prev_char (&text[*start]);
+        while (ti > text && (ti[0] == ' ' || ti[0] == '\t'))
+            str_prev_char (&ti);
+        if (ti <= text&& (ti[0] == ' ' || ti[0] == '\t'))
+            in_command_position++;
+        else if (strchr (command_separator_chars, ti[0])){
             register int this_char, prev_char;
 
             in_command_position++;
-            
-            if (i){
+
+            if (ti > text){
                 /* Handle the two character tokens `>&', `<&', and `>|'.
                    We are not in a command position after one of these. */
-                this_char = text[i];
-                prev_char = text[i - 1];
+                this_char = ti[0];
+                prev_char = str_get_prev_char (ti)[0];
 
                 if ((this_char == '&' && (prev_char == '<' || prev_char == '>')) ||
 	            (this_char == '|' && prev_char == '>'))
 	            in_command_position = 0;
-                else if (i > 0 && text [i-1] == '\\') /* Quoted */
+
+            else if (ti > text && str_get_prev_char (ti)[0] == '\\') /* Quoted */
 	            in_command_position = 0;
 	    }
 	}
@@ -726,17 +755,19 @@ try_complete (char *text, int *start, int *end, INPUT_COMPLETE_FLAGS flags)
     if (flags & INPUT_COMPLETE_HOSTNAMES)    
         r = strrchr (word, '@');
     if (q && q [1] == '(' && INPUT_COMPLETE_COMMANDS){
-    	if (q > p)
-    	    p = q + 1;
-    	q = NULL;
+	if (q > p)
+	    p = str_get_next_char (q);
+	q = NULL;
     }
 
     /* Command substitution? */
     if (p > q && p > r){
         SHOW_C_CTX("try_complete:cmd_backq_subst");
-        matches = completion_matches (p + 1, command_completion_function, flags & (~INPUT_COMPLETE_FILENAMES));
+        matches = completion_matches (str_cget_next_char (p),
+                                      command_completion_function, 
+                                      flags & (~INPUT_COMPLETE_FILENAMES));
         if (matches)
-            *start += p + 1 - word;
+            *start += str_get_next_char (p) - word;
     }
 
     /* Variable name? */
@@ -755,7 +786,7 @@ try_complete (char *text, int *start, int *end, INPUT_COMPLETE_FLAGS flags)
         if (matches)
             *start += r - word;
     }
-        
+
     /* Starts with `~' and there is no slash in the word, then
        try completing this word as a username. */
     if (!matches && *word == '~' && (flags & INPUT_COMPLETE_USERNAMES) && !strchr (word, PATH_SEP))
@@ -773,36 +804,21 @@ try_complete (char *text, int *start, int *end, INPUT_COMPLETE_FLAGS flags)
         SHOW_C_CTX("try_complete:cmd_subst");
         matches = completion_matches (word, command_completion_function, flags & (~INPUT_COMPLETE_FILENAMES));
     }
-        
+
     else if (!matches && (flags & INPUT_COMPLETE_FILENAMES)){
 	if (is_cd)
 	    flags &= ~(INPUT_COMPLETE_FILENAMES | INPUT_COMPLETE_COMMANDS);
 	SHOW_C_CTX("try_complete:filename_subst_1");
 	matches = completion_matches (word, filename_completion_function, flags);
-    	if (!matches && is_cd && *word != PATH_SEP && *word != '~'){
-    	    char *p, *q = text + *start;
-
-	    for (p = text; *p && p < q; p++){
-		if (*p == ' ' || *p == '\t') {
-		    if (p == text) continue;
-		    if (*(p-1) == '\\') {
-			p--;
-			break;
-		    }
-		}
-	    }
-    	    if (!strncmp (p, "cd", 2))
-		for (p += 2; *p && p < q && (*p == ' ' || *p == '\t'); p++){
-		    if (p == text) continue;
-		    if (*(p-1) == '\\') {
-			p--;
-			break;
-		    }
-		}
-    	    if (p == q){
+	if (!matches && is_cd && *word != PATH_SEP && *word != '~'){
+	    q = text + *start;
+	    for (p = text; *p && p < q && (*p == ' ' || *p == '\t'); str_next_char (&p));
+	    if (!strncmp (p, "cd", 2))
+	        for (p += 2; *p && p < q && (*p == ' ' || *p == '\t'); str_next_char (&p));
+	    if (p == q){
 		char * const cdpath_ref = g_strdup (getenv ("CDPATH"));
 		char *cdpath = cdpath_ref;
-		char c, *s, *r;
+		char c, *s;
 
 		if (cdpath == NULL)
 		    c = 0;
@@ -815,17 +831,17 @@ try_complete (char *text, int *start, int *end, INPUT_COMPLETE_FLAGS flags)
 		    c = *s; 
 		    *s = 0;
 		    if (*cdpath){
-			r = mhl_str_dir_plus_file (cdpath, word);
+			r = concat_dir_and_file (cdpath, word);
 			SHOW_C_CTX("try_complete:filename_subst_2");
 			matches = completion_matches (r, filename_completion_function, flags);
 			g_free (r);
 		    }
 		    *s = c;
-		    cdpath = s + 1;
+		    cdpath = str_get_next_char (s);
 		}
 		g_free (cdpath_ref);
-    	    }
-    	}
+	    }
+	}
     }
 
     g_free (word);
@@ -836,7 +852,7 @@ try_complete (char *text, int *start, int *end, INPUT_COMPLETE_FLAGS flags)
 void free_completions (WInput *in)
 {
     char **p;
-    
+
     if (!in->completions)
     	return;
     for (p=in->completions; *p; p++)
@@ -850,49 +866,59 @@ static WInput *input;
 static int min_end;
 static int start, end;
 
-static int insert_text (WInput *in, char *text, ssize_t len)
+static int
+insert_text (WInput *in, char *text, ssize_t size)
 {
-    len = min (len, (ssize_t) strlen (text)) + start - end;
-    if (strlen (in->buffer) + len >= (size_t) in->current_max_len){
+    int buff_len = str_length (in->buffer);
+
+    size = min (size, (ssize_t) strlen (text)) + start - end;
+    if (strlen (in->buffer) + size >= (size_t) in->current_max_size){
     /* Expand the buffer */
-    	char *narea = g_realloc (in->buffer, in->current_max_len + len + in->field_len);
-	if (narea){
+	char *narea = g_try_realloc (in->buffer, in->current_max_size
+					+ size + in->field_width);
+	if (narea != NULL) {
 	    in->buffer = narea;
-	    in->current_max_len += len + in->field_len;
+	    in->current_max_size += size + in->field_width;
 	}
     }
-    if (strlen (in->buffer)+1 < (size_t) in->current_max_len){
-    	if (len > 0){
+    if (strlen (in->buffer)+1 < (size_t) in->current_max_size){
+    	if (size > 0){
 	    int i = strlen (&in->buffer [end]);
 	    for (; i >= 0; i--)
-	        in->buffer [end + len + i] = in->buffer [end + i];
-	} else if (len < 0){
-	    char *p = in->buffer + end + len, *q = in->buffer + end;
+	        in->buffer [end + size + i] = in->buffer [end + i];
+	} else if (size < 0){
+	    char *p = in->buffer + end + size, *q = in->buffer + end;
 	    while (*q)
 	    	*(p++) = *(q++);
 	    *p = 0;
 	}
-	memcpy (in->buffer + start, text, len - start + end);
-	in->point += len;
+	memcpy (in->buffer + start, text, size - start + end);
+	in->point+= str_length (in->buffer) - buff_len;
 	update_input (in, 1);
-	end += len;
+	end+= size;
     }
-    return len != 0;
+    return size != 0;
 }
 
 static cb_ret_t
-query_callback (Dlg_head *h, dlg_msg_t msg, int parm)
+query_callback (Dlg_head *h, Widget *sender,
+		dlg_msg_t msg, int parm, void *data)
 {
+    static char buff[MB_LEN_MAX] = "";
+    static int bl = 0;
+
     switch (msg) {
     case DLG_KEY:
 	switch (parm) {
 	case KEY_LEFT:
 	case KEY_RIGHT:
+            bl = 0;
 	    h->ret_value = 0;
 	    dlg_stop (h);
 	    return MSG_HANDLED;
 
 	case KEY_BACKSPACE:
+            bl = 0;
 	    if (end == min_end) {
 		h->ret_value = 0;
 		dlg_stop (h);
@@ -902,13 +928,13 @@ query_callback (Dlg_head *h, dlg_msg_t msg, int parm)
 
 		e1 = e = ((WListbox *) (h->current))->list;
 		do {
-		    if (!strncmp
-			(input->buffer + start, e1->text,
-			 end - start - 1)) {
-			listbox_select_entry ((WListbox *) (h->current),
-					      e1);
+		    if (!strncmp (input->buffer + start, 
+                         e1->text, end - start - 1)) {
+                                 
+			listbox_select_entry ((WListbox *) (h->current), e1);
+			end = str_get_prev_char (&(input->buffer[end])) 
+                                - input->buffer;
 			handle_char (input, parm);
-			end--;
 			send_message (h->current, WIDGET_DRAW, 0);
 			break;
 		    }
@@ -918,7 +944,8 @@ query_callback (Dlg_head *h, dlg_msg_t msg, int parm)
 	    return MSG_HANDLED;
 
 	default:
-	    if (parm > 0xff || !is_printable (parm)) {
+	    if (parm < 32 || parm > 256) {
+                bl = 0;
 		if (is_in_input_map (input, parm) == 2) {
 		    if (end == min_end)
 			return MSG_HANDLED;
@@ -934,21 +961,43 @@ query_callback (Dlg_head *h, dlg_msg_t msg, int parm)
 		int low = 4096;
 		char *last_text = NULL;
 
+                buff[bl] = (char) parm;
+                bl++;
+                buff[bl] = '\0';
+                switch (str_is_valid_char (buff, bl)) {
+                    case -1: 
+                        bl = 0;
+                    case -2:
+                        return MSG_HANDLED;
+                }
+                
 		e1 = e = ((WListbox *) (h->current))->list;
 		do {
-		    if (!strncmp
-			(input->buffer + start, e1->text, end - start)) {
-			if (e1->text[end - start] == parm) {
+		    if (!strncmp (input->buffer + start, 
+                         e1->text, end - start)) {
+			
+                        if (strncmp (&e1->text[end - start], buff, bl) == 0) {
 			    if (need_redraw) {
-				register int c1, c2, si;
+				char *si, *sl;
+				char *ni, *nl;
+                                si = &(e1->text[end - start]);
+                                sl = &(last_text[end - start]);
+                                
+				for (; si[0] != '\0' && sl[0] != '\0';) {
+                                    
+                                    ni = str_get_next_char (si);
+                                    nl = str_get_next_char (sl);
+                                    
+                                    if (ni - si != nl - sl) break;
+                                    if (strncmp (si, sl, ni - si) != 0) break;
+                                    
+                                    si = ni;
+                                    sl = nl;
+                                }
+                                
+				if (low > si - &e1->text[end - start])
+				    low = si - &e1->text[end - start];
 
-				for (si = end - start + 1;
-				     (c1 = last_text[si])
-				     && (c2 = e1->text[si]); si++)
-				    if (c1 != c2)
-					break;
-				if (low > si)
-				    low = si;
 				last_text = e1->text;
 				need_redraw = 2;
 			    } else {
@@ -969,13 +1018,14 @@ query_callback (Dlg_head *h, dlg_msg_t msg, int parm)
 		    h->ret_value = B_ENTER;
 		    dlg_stop (h);
 		}
+                bl = 0;
 	    }
 	    return MSG_HANDLED;
 	}
 	break;
 
     default:
-	return default_dlg_callback (h, msg, parm);
+	return default_dlg_callback (h, sender, msg, parm, data);
     }
 }
 
@@ -985,94 +1035,96 @@ query_callback (Dlg_head *h, dlg_msg_t msg, int parm)
 static int
 complete_engine (WInput *in, int what_to_do)
 {
-    if (in->completions && in->point != end)
+    int s;
+
+    if (in->completions && (str_offset_to_pos (in->buffer, in->point)) != end)
     	free_completions (in);
     if (!in->completions){
-    	end = in->point;
-        for (start = end ? end - 1 : 0; start > -1; start--)
-    	    if (strchr (" \t;|<>", in->buffer [start])){
-    		if (start > 0 && in->buffer [start-1] == '\\')
-    		    continue;
-    		else
-    		    break;
-    	    }
-    	if (start < end)
-    	    start++;
-    	in->completions = try_complete (in->buffer, &start, &end, in->completion_flags);
+        end = str_offset_to_pos (in->buffer, in->point);
+        for (s = in->point ? in->point - 1 : 0; s >= 0; s--) {
+            start = str_offset_to_pos (in->buffer, s);
+            if (strchr (" \t;|<>", in->buffer [start])) {
+                if (start < end) start = str_offset_to_pos (in->buffer, s + 1);
+                /* FIXME: maybe need check '\\' prev char
+                   if (start > 0 && in->buffer [start-1] == '\\')
+                */
+                break;
+            }
+        }
+        in->completions = try_complete (in->buffer, &start, &end, in->completion_flags);
     }
-    
+
     if (in->completions){
-    	if (what_to_do & DO_INSERTION || ((what_to_do & DO_QUERY) && !in->completions[1])) {
-	        char * complete = in->completions [0];
-	    if (insert_text (in, complete, strlen (complete))){
-    	        if (in->completions [1])
-    	    	    beep ();
+	if (what_to_do & DO_INSERTION || ((what_to_do & DO_QUERY) && !in->completions[1])) {
+	    char * lc_complete = in->completions [0];
+	    if (insert_text (in, lc_complete, strlen (lc_complete))){
+	        if (in->completions [1])
+		    tty_beep ();
 		else
 		    free_completions (in);
 	    } else
-	        beep ();
+		tty_beep ();
         }
-    	if ((what_to_do & DO_QUERY) && in->completions && in->completions [1]) {
-    	    int maxlen = 0, i, count = 0;
-    	    int x, y, w, h;
-    	    int start_x, start_y;
-    	    char **p, *q;
-    	    Dlg_head *query_dlg;
-    	    WListbox *query_list;
+	if ((what_to_do & DO_QUERY) && in->completions && in->completions [1]) {
+	    int maxlen = 0, i, count = 0;
+	    int x, y, w, h;
+	    int start_x, start_y;
+	    char **p, *q;
+	    Dlg_head *query_dlg;
+	    WListbox *query_list;
 
-    	    for (p=in->completions + 1; *p; count++, p++) {
-		if ((i = strlen (*p)) > maxlen)
-		    maxlen = i;
+	    for (p=in->completions + 1; *p; count++, p++)
+	        if ((i = str_term_width1 (*p)) > maxlen)
+	            maxlen = i;
+	    start_x = in->widget.x;
+	    start_y = in->widget.y;
+	    if (start_y - 2 >= count) {
+	    	y = start_y - 2 - count;
+	    	h = 2 + count;
+	    } else {
+	        if (start_y >= LINES - start_y - 1) {
+	            y = 0;
+	            h = start_y;
+	        } else {
+	            y = start_y + 1;
+	            h = LINES - start_y - 1;
+	        }
 	    }
-    	    start_x = in->widget.x;
-    	    start_y = in->widget.y;
-    	    if (start_y - 2 >= count) {
-    	    	y = start_y - 2 - count;
-    	    	h = 2 + count;
-    	    } else {
-    	    	if (start_y >= LINES - start_y - 1) {
-    	    	    y = 0;
-    	    	    h = start_y;
-    	    	} else {
-    	    	    y = start_y + 1;
-    	    	    h = LINES - start_y - 1;
-    	    	}
-    	    }
-    	    x = start - in->first_shown - 2 + start_x;
-    	    w = maxlen + 4;
-    	    if (x + w > COLS)
-    	    	x = COLS - w;
-    	    if (x < 0)
-    	    	x = 0;
-    	    if (x + w > COLS)
-    	    	w = COLS;
-    	    input = in;
-    	    min_end = end;
+	    x = start - in->term_first_shown - 2 + start_x;
+	    w = maxlen + 4;
+	    if (x + w > COLS)
+	        x = COLS - w;
+	    if (x < 0)
+	        x = 0;
+	    if (x + w > COLS)
+	        w = COLS;
+	    input = in;
+	    min_end = end;
 	    query_height = h;
 	    query_width  = w;
-    	    query_dlg = create_dlg (y, x, query_height, query_width,
+	    query_dlg = create_dlg (y, x, query_height, query_width,
 				    dialog_colors, query_callback,
 				    "[Completion]", NULL, DLG_COMPACT);
-    	    query_list = listbox_new (1, 1, w - 2, h - 2, NULL);
-    	    add_widget (query_dlg, query_list);
-    	    for (p = in->completions + 1; *p; p++)
-    	    	listbox_add_item (query_list, 0, 0, *p, NULL);
-    	    run_dlg (query_dlg);
-    	    q = NULL;
-    	    if (query_dlg->ret_value == B_ENTER){
-    	    	listbox_get_current (query_list, &q, NULL);
-    	    	if (q)
-    	    	    insert_text (in, q, strlen (q));
-    	    }
-    	    if (q || end != min_end)
-    	    	free_completions (in);
-    	    i = query_dlg->ret_value; /* B_USER if user wants to start over again */
-    	    destroy_dlg (query_dlg);
-    	    if (i == B_USER)
-    	    	return 1;
-    	}
+	    query_list = listbox_new (1, 1, h - 2, w - 2, NULL);
+	    add_widget (query_dlg, query_list);
+	    for (p = in->completions + 1; *p; p++)
+		listbox_add_item (query_list, 0, 0, *p, NULL);
+	    run_dlg (query_dlg);
+	    q = NULL;
+	    if (query_dlg->ret_value == B_ENTER){
+		listbox_get_current (query_list, &q, NULL);
+		if (q)
+		    insert_text (in, q, strlen (q));
+	    }
+	    if (q || end != min_end)
+		free_completions (in);
+	    i = query_dlg->ret_value; /* B_USER if user wants to start over again */
+	    destroy_dlg (query_dlg);
+	    if (i == B_USER)
+		return 1;
+	}
     } else
-    	beep ();
+	tty_beep ();
     return 0;
 }
 
@@ -1080,6 +1132,8 @@ void complete (WInput *in)
 {
     int engine_flags;
 
+    if (!str_is_valid_string (in->buffer)) return;
+
     if (in->completions)
 	engine_flags = DO_QUERY;
     else
