@@ -22,14 +22,23 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+
+/** \file  filenot.c
+ *  \brief Source: wrapper for routines to notify the
+ *  tree about the changes made to the directory
+ *  structure.
+ */
+
 #include <config.h>
 
 #include <errno.h>
 #include <string.h>
 
-#include <mhl/string.h>
-
 #include "global.h"
+#include "fs.h"
+#include "util.h"
+
+#include "../vfs/vfs.h"
 
 static char *
 get_absolute_name (const char *file)
@@ -39,7 +48,7 @@ get_absolute_name (const char *file)
     if (file[0] == PATH_SEP)
 	return g_strdup (file);
     mc_get_current_wd (dir, MC_MAXPATHLEN);
-    return mhl_str_dir_plus_file (dir, file);
+    return concat_dir_and_file (dir, file);
 }
 
 static int
@@ -62,7 +71,7 @@ my_mkdir_rec (char *s, mode_t mode)
 	return -1;
     }
 
-    p = mhl_str_dir_plus_file (s, "..");
+    p = concat_dir_and_file (s, "..");
     q = vfs_canon (p);
     g_free (p);
 
