@@ -1,8 +1,15 @@
+
+/** \file panel.h
+ *  \brief Header: defines WPanel structure
+ */
+
 #ifndef MC_PANEL_H
 #define MC_PANEL_H
 
 #include "dir.h"		/* dir_list */
 #include "dialog.h"		/* Widget */
+#include "fs.h"			/* MC_MAXPATHLEN */
+#include "strutil.h"
 
 #define selection(p) (&(p->dir.list[p->selected]))
 #define DEFAULT_USER_FORMAT "half type name | size | perm"
@@ -31,6 +38,21 @@ enum panel_display_enum {
 
 struct format_e;
 
+typedef struct panel_format_struct {
+    const char *id;
+    int  min_size;
+    int  expands;
+    align_crt_t default_just;
+    const char *hotkey;
+    const char *title_hotkey;
+    gboolean is_user_choice;
+    gboolean use_in_user_format;
+    const char *(*string_fn)(file_entry *, int);
+    sortfn *sort_routine; /* used by mouse_sort_col() */
+} panel_field_t;
+
+extern panel_field_t panel_fields [];
+
 typedef struct WPanel {
     Widget   widget;
     dir_list dir;		/* Directory contents */
@@ -53,7 +75,7 @@ typedef struct WPanel {
     int      split;		/* Split panel to allow two columns */
     int      is_panelized;	/* Flag: special filelisting, can't reload */
     int      frame_size;	/* half or full frame */
-    sortfn   *sort_type;	/* Sort type */
+    const panel_field_t *current_sort_field;
     char     *filter;		/* File name filter */
 
     int      dirty;		/* Should we redisplay the panel? */
@@ -72,9 +94,12 @@ typedef struct WPanel {
 
     int      searching;
     char     search_buffer [256];
+    char     search_char [MB_LEN_MAX]; /*buffer for multibytes characters*/
+    int      search_chpoint;           /*point after last characters in search_char*/
 } WPanel;
 
 WPanel *panel_new (const char *panel_name);
+WPanel *panel_new_with_dir (const char *panel_name, const char *dr);
 void panel_clean_dir (WPanel *panel);
 
 extern int torben_fj_mode;
@@ -85,10 +110,18 @@ extern int panel_scroll_pages;
 extern int fast_reload;
 
 void panel_reload         (WPanel *panel);
-void panel_set_sort_order (WPanel *panel, sortfn *sort_order);
+void panel_set_sort_order (WPanel *panel, const panel_field_t *sort_order);
 void panel_re_sort        (WPanel *panel);
+void set_panel_encoding (WPanel *);
+
+#define UP_OPTIMIZE		0
+#define UP_RELOAD		1
+#define UP_ONLY_CURRENT		2
+
+#define UP_KEEPSEL ((char *) -1)
 
 void update_dirty_panels (void);
+void update_panels (int force_update, const char *current_file);
 void panel_update_cols (Widget *widget, int frame_size);
 int set_panel_formats (WPanel *p);
 
@@ -113,4 +146,15 @@ void directory_history_next (WPanel *panel);
 void directory_history_prev (WPanel *panel);
 void directory_history_list (WPanel *panel);
 
+gsize panel_get_num_of_sortable_fields(void);
+const char **panel_get_sortable_fields(gsize *);
+const panel_field_t *panel_get_field_by_id(const char *);
+const panel_field_t *panel_get_field_by_title(const char *);
+const panel_field_t *panel_get_field_by_title_hotkey(const char *);
+gsize panel_get_num_of_user_possible_fields(void);
+const char **panel_get_user_possible_fields(gsize *);
+
+void panel_init(void);
+void panel_deinit(void);
+
 #endif
