@@ -37,7 +37,7 @@
 
 /*** file scope variables ************************************************************************/
 
-static const char ESCAPE_SHELL_CHARS[] = " !#$%()&{}[]`?|<>;*\\\"";
+static const char ESCAPE_SHELL_CHARS[] = " !#$%()&{}[]`?|<>;*\\\"'";
 static const char ESCAPE_REGEX_CHARS[] = "^!#$%()&{}[]`?|<>;*.\\";
 static const char ESCAPE_GLOB_CHARS[]  = "$*\\";
 
