@@ -1,7 +1,18 @@
+
+/** \file util.h
+ *  \brief Header: various utilities
+ */
+
 #ifndef MC_UTIL_H
 #define MC_UTIL_H
 
+#include "global.h"		/* include <glib.h> */
+
+#include <string.h>
+#include <assert.h>
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 /* Returns its argument as a "modifiable" string. This function is
  * intended to pass strings to legacy libraries that don't know yet
@@ -34,12 +45,12 @@ char *fake_name_quote (const char *c, int quote_percent);
 /* Remove the middle part of the string to fit given length.
  * Use "~" to show where the string was truncated.
  * Return static buffer, no need to free() it. */
-const char *name_trunc (const char *txt, int trunc_len);
+const char *name_trunc (const char *txt, size_t trunc_len);
 
 /* path_trunc() is the same as name_trunc() above but
  * it deletes possible password from path for security
  * reasons. */
-const char *path_trunc (const char *path, int trunc_len);
+const char *path_trunc (const char *path, size_t trunc_len);
 
 /* return a static string representing size, appending "K" or "M" for
  * big sizes.
@@ -55,7 +66,7 @@ const char *size_trunc_sep (double size);
  * not including trailing 0. BUFFER should be at least LEN+1 long.
  *
  * Units: size units (0=bytes, 1=Kbytes, 2=Mbytes, etc.) */
-void size_trunc_len (char *buffer, int len, off_t size, int units);
+void size_trunc_len (char *buffer, unsigned int len, off_t size, int units);
 int  is_exe (mode_t mode);
 const char *string_perm (mode_t mode_bits);
 
@@ -66,6 +77,7 @@ char *strip_password (char *path, int has_prefix);
 const char *strip_home_and_password (const char *dir);
 
 const char *extension (const char *);
+char *concat_dir_and_file (const char *dir, const char *file);
 const char *unix_error_string (int error_num);
 const char *skip_separators (const char *s);
 const char *skip_numbers (const char *s);
@@ -84,12 +96,6 @@ char *diff_two_paths (const char *first, const char *second);
 /* Returns the basename of fname. The result is a pointer into fname. */
 const char *x_basename (const char *fname);
 
-/* Profile managing functions */
-int set_int (const char *, const char *, int);
-int get_int (const char *, const char *, int);
-extern char * get_config_string (const char *, const char *, const char *);
-extern void set_config_string (const char *, const char *, const char *);
-
 char *load_file (const char *filename);
 char *load_mc_home_file (const char *filename, char ** allocated_filename);
 
@@ -111,10 +117,11 @@ const char *file_date (time_t);
 
 int exist_file (const char *name);
 
+/* Check if the file exists. If not copy the default */
+int check_for_default (const char *default_file, const char *file);
+
 /* Returns a copy of *s until a \n is found and is below top */
 const char *extract_line (const char *s, const char *top);
-const char *_icase_search (const char *text, const char *data, int *lng);
-#define icase_search(T,D) _icase_search((T), (D), NULL)
 
 /* Matching */
 enum {
@@ -124,8 +131,6 @@ enum {
 };
 
 extern int easy_patterns;
-char *convert_pattern (const char *pattern, int match_type, int do_group);
-int regexp_match (const char *pattern, const char *string, int match_type);
 
 /* Error pipes */
 void open_error_pipe (void);
@@ -141,6 +146,17 @@ extern struct sigaction startup_handler;
 char *tilde_expand (const char *);
 
 /* Pathname canonicalization */
+typedef enum {
+    CANON_PATH_JOINSLASHES	= 1L<<0, /* Multiple `/'s are collapsed to a single `/'. */
+    CANON_PATH_REMSLASHDOTS	= 1L<<1, /* Leading `./'s, `/'s and trailing `/.'s are removed. */
+    CANON_PATH_REMDOUBLEDOTS	= 1L<<3, /* Non-leading `../'s and trailing `..'s are handled by removing */
+    CANON_PATH_GUARDUNC		= 1L<<4, /* Detect and preserve UNC paths: //server/... */
+    CANON_PATH_ALL	= CANON_PATH_JOINSLASHES
+			| CANON_PATH_REMSLASHDOTS
+			| CANON_PATH_REMDOUBLEDOTS
+			| CANON_PATH_GUARDUNC
+} CANON_PATH_FLAGS;
+void custom_canonicalize_pathname (char *, CANON_PATH_FLAGS);
 void canonicalize_pathname (char *);
 
 /* Misc Unix functions */
@@ -178,12 +194,14 @@ enum compression_type {
 	COMPRESSION_NONE,
 	COMPRESSION_GZIP,
 	COMPRESSION_BZIP,
-	COMPRESSION_BZIP2
+	COMPRESSION_BZIP2,
+	COMPRESSION_LZMA,
+	COMPRESSION_XZ
 };
 
 /* Looks for ``magic'' bytes at the start of the VFS file to guess the
  * compression type. Side effect: modifies the file position. */
-enum compression_type get_compression_type (int fd);
+enum compression_type get_compression_type (int fd, const char*);
 const char *decompress_extension (int type);
 
 /* Hook functions */
@@ -203,12 +221,6 @@ GList *list_append_unique (GList *list, char *text);
 
 /* Position saving and restoring */
 
-/* file where positions are stored */
-#define MC_FILEPOS ".mc/filepos"
-/* temporary file */
-#define MC_FILEPOS_TMP ".mc/filepos.tmp"
-/* maximum entries in MC_FILEPOS */
-#define MC_FILEPOS_ENTRIES 1024
 /* Load position for the given filename */
 void load_file_position (const char *filename, long *line, long *column);
 /* Save position for the given filename */
@@ -255,4 +267,34 @@ extern int ascii_alpha_to_cntrl (int ch);
 #undef Q_
 const char *Q_ (const char *s);
 
+#define str_dup_range(s_start, s_bound) (g_strndup(s_start, s_bound - s_start))
+
+/*
+ * strcpy is unsafe on overlapping memory areas, so define memmove-alike
+ * string function.
+ * Have sense only when:
+ *  * dest <= src
+ *   AND
+ *  * dest and str are pointers to one object (as Roland Illig pointed).
+ *
+ * We can't use str*cpy funs here:
+ * http://kerneltrap.org/mailarchive/openbsd-misc/2008/5/27/1951294
+ */
+static inline char * str_move(char * dest, const char * src)
+{
+    size_t n;
+
+    assert (dest<=src);
+
+    n = strlen (src) + 1; /* + '\0' */
+
+    return memmove (dest, src, n);
+}
+
+gboolean mc_util_make_backup_if_possible (const char *, const char *);
+gboolean mc_util_restore_from_backup_if_possible (const char *, const char *);
+gboolean mc_util_unlink_backup_if_possible (const char *, const char *);
+
+#define MC_PTR_FREE(ptr) do { g_free(ptr); (ptr) = NULL; } while (0)
+
 #endif
