@@ -22,6 +22,10 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
+/** \file utilunix.c
+ *  \brief Source: various utilities - Unix variant
+ */
+
 #include <config.h>
 
 #include <ctype.h>
@@ -32,16 +36,17 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-
+#include <fcntl.h>
 #include <sys/param.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #ifdef HAVE_SYS_IOCTL_H
 #   include <sys/ioctl.h>
 #endif
 #include <unistd.h>
-
-#include <mhl/string.h>
+#include <pwd.h>
+#include <grp.h>
 
 #include "global.h"
 #include "execute.h"
@@ -158,7 +163,26 @@ int my_system (int flags, const char *shell, const char *command)
 	if (flags & EXECUTE_AS_SHELL)
 	    execl (shell, shell, "-c", command, (char *) NULL);
 	else
-	    execlp (shell, shell, command, (char *) NULL);
+	{
+	    gchar **shell_tokens;
+	    const gchar *only_cmd;
+	    shell_tokens = g_strsplit(shell," ", 2);
+
+	    if (shell_tokens == NULL)
+	        only_cmd = shell;
+	    else
+	        only_cmd = (*shell_tokens) ? *shell_tokens: shell;
+
+	    execlp (only_cmd, shell, command, (char *) NULL);
+
+	    /*
+	      execlp will replace current process,
+	      therefore no sence in call of g_strfreev().
+	      But this keeped for estetic reason :)
+	    */
+	    g_strfreev(shell_tokens);
+
+	}
 
 	_exit (127);		/* Exec error */
     } else {
@@ -341,15 +365,39 @@ static int old_error;		/* File descriptor of old standard error */
 void open_error_pipe (void)
 {
     if (pipe (error_pipe) < 0){
-	message (0, _("Warning"), _(" Pipe failed "));
+	message (D_NORMAL, _("Warning"), _(" Pipe failed "));
     }
     old_error = dup (2);
     if(old_error < 0 || close(2) || dup (error_pipe[1]) != 2){
-	message (0, _("Warning"), _(" Dup failed "));
+	message (D_NORMAL, _("Warning"), _(" Dup failed "));
+
 	close (error_pipe[0]);
-	close (error_pipe[1]);
+	error_pipe[0] = -1;
     }
+    else
+    {
+        /*
+         * Settng stderr in nonblocking mode as we close it earlier, than
+         * program stops. We try to read some error at program startup,
+         * but we should not block on it.
+         *
+         * TODO: make piped stdin/stderr poll()/select()able to get rid
+         * of following hack.
+         */
+        int fd_flags;
+        fd_flags = fcntl (error_pipe[0], F_GETFL, NULL);
+        if (fd_flags != -1)
+        {
+            fd_flags |= O_NONBLOCK;
+            if (fcntl(error_pipe[0], F_SETFL, fd_flags) == -1)
+            {
+                /* TODO: handle it somehow */
+            }
+        }
+    }
+    /* we never write there */
     close (error_pipe[1]);
+    error_pipe[1] = -1;
 }
 
 /*
@@ -364,6 +412,10 @@ close_error_pipe (int error, const char *text)
     char msg[MAX_PIPE_SIZE];
     int len = 0;
 
+    /* already closed */
+    if (error_pipe[0] == -1)
+	return 0;
+
     if (error)
 	title = MSG_ERROR;
     else
@@ -377,6 +429,7 @@ close_error_pipe (int error, const char *text)
 	if (len >= 0)
 	    msg[len] = 0;
 	close (error_pipe[0]);
+	error_pipe[0] = -1;
     }
     if (error < 0)
 	return 0;	/* Just ignore error message */
@@ -404,14 +457,14 @@ close_error_pipe (int error, const char *text)
  * Well formed UNC paths are modified only in the local part.
  */
 void
-canonicalize_pathname (char *path)
+custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
 {
     char *p, *s;
     int len;
     char *lpath = path;	/* path without leading UNC part */
 
     /* Detect and preserve UNC paths: //server/... */
-    if (path[0] == PATH_SEP && path[1] == PATH_SEP) {
+    if ( ( flags & CANON_PATH_GUARDUNC ) && path[0] == PATH_SEP && path[1] == PATH_SEP) {
 	p = path + 2;
 	while (p[0] && p[0] != '/')
 	    p++;
@@ -422,112 +475,126 @@ canonicalize_pathname (char *path)
     if (!lpath[0] || !lpath[1])
 	return;
 
-    /* Collapse multiple slashes */
-    p = lpath;
-    while (*p) {
-	if (p[0] == PATH_SEP && p[1] == PATH_SEP) {
-	    s = p + 1;
-	    while (*(++s) == PATH_SEP);
-	    mhl_strmove (p + 1, s);
-	}
-	p++;
-    }
-
-    /* Collapse "/./" -> "/" */
-    p = lpath;
-    while (*p) {
-	if (p[0] == PATH_SEP && p[1] == '.' && p[2] == PATH_SEP)
-	    mhl_strmove (p, p + 2);
-	else
+    if ( flags & CANON_PATH_JOINSLASHES ) {
+	/* Collapse multiple slashes */
+	p = lpath;
+	while (*p) {
+	    if (p[0] == PATH_SEP && p[1] == PATH_SEP) {
+		s = p + 1;
+		while (*(++s) == PATH_SEP);
+		str_move (p + 1, s);
+	    }
 	    p++;
-    }
-
-    /* Remove trailing slashes */
-    p = lpath + strlen (lpath) - 1;
-    while (p > lpath && *p == PATH_SEP)
-	*p-- = 0;
-
-    /* Remove leading "./" */
-    if (lpath[0] == '.' && lpath[1] == PATH_SEP) {
-	if (lpath[2] == 0) {
-	    lpath[1] = 0;
-	    return;
-	} else {
-	    mhl_strmove (lpath, lpath + 2);
 	}
     }
 
-    /* Remove trailing "/" or "/." */
-    len = strlen (lpath);
-    if (len < 2)
-	return;
-    if (lpath[len - 1] == PATH_SEP) {
-	lpath[len - 1] = 0;
-    } else {
-	if (lpath[len - 1] == '.' && lpath[len - 2] == PATH_SEP) {
-	    if (len == 2) {
+    if ( flags & CANON_PATH_JOINSLASHES ) {
+	/* Collapse "/./" -> "/" */
+	p = lpath;
+	while (*p) {
+	    if (p[0] == PATH_SEP && p[1] == '.' && p[2] == PATH_SEP)
+		str_move (p, p + 2);
+	    else
+		p++;
+	}
+    }
+
+    if ( flags & CANON_PATH_REMSLASHDOTS ) {
+	/* Remove trailing slashes */
+	p = lpath + strlen (lpath) - 1;
+	while (p > lpath && *p == PATH_SEP)
+	    *p-- = 0;
+
+	/* Remove leading "./" */
+	if (lpath[0] == '.' && lpath[1] == PATH_SEP) {
+	    if (lpath[2] == 0) {
 		lpath[1] = 0;
 		return;
 	    } else {
-		lpath[len - 2] = 0;
+		str_move (lpath, lpath + 2);
+	    }
+	}
+
+	/* Remove trailing "/" or "/." */
+	len = strlen (lpath);
+	if (len < 2)
+	    return;
+	if (lpath[len - 1] == PATH_SEP) {
+	    lpath[len - 1] = 0;
+	} else {
+	    if (lpath[len - 1] == '.' && lpath[len - 2] == PATH_SEP) {
+		if (len == 2) {
+		    lpath[1] = 0;
+		    return;
+		} else {
+		    lpath[len - 2] = 0;
+		}
 	    }
 	}
     }
 
-    /* Collapse "/.." with the previous part of path */
-    p = lpath;
-    while (p[0] && p[1] && p[2]) {
-	if ((p[0] != PATH_SEP || p[1] != '.' || p[2] != '.')
-	    || (p[3] != PATH_SEP && p[3] != 0)) {
-	    p++;
-	    continue;
-	}
+    if ( flags & CANON_PATH_REMDOUBLEDOTS ) {
+	/* Collapse "/.." with the previous part of path */
+	p = lpath;
+	while (p[0] && p[1] && p[2]) {
+	    if ((p[0] != PATH_SEP || p[1] != '.' || p[2] != '.')
+		|| (p[3] != PATH_SEP && p[3] != 0)) {
+		p++;
+		continue;
+	    }
 
-	/* search for the previous token */
-	s = p - 1;
-	while (s >= lpath && *s != PATH_SEP)
-	    s--;
+	    /* search for the previous token */
+	    s = p - 1;
+	    while (s >= lpath && *s != PATH_SEP)
+		s--;
 
-	s++;
+	    s++;
 
-	/* If the previous token is "..", we cannot collapse it */
-	if (s[0] == '.' && s[1] == '.' && s + 2 == p) {
-	    p += 3;
-	    continue;
-	}
+	    /* If the previous token is "..", we cannot collapse it */
+	    if (s[0] == '.' && s[1] == '.' && s + 2 == p) {
+		p += 3;
+		continue;
+	    }
 
-	if (p[3] != 0) {
-	    if (s == lpath && *s == PATH_SEP) {
-		/* "/../foo" -> "/foo" */
-		mhl_strmove (s + 1, p + 4);
+	    if (p[3] != 0) {
+		if (s == lpath && *s == PATH_SEP) {
+		    /* "/../foo" -> "/foo" */
+		    str_move (s + 1, p + 4);
+		} else {
+		    /* "token/../foo" -> "foo" */
+		    str_move (s, p + 4);
+		}
+		p = (s > lpath) ? s - 1 : s;
+		continue;
+	    }
+
+	    /* trailing ".." */
+	    if (s == lpath) {
+		/* "token/.." -> "." */
+		if (lpath[0] != PATH_SEP) {
+		    lpath[0] = '.';
+		}
+		lpath[1] = 0;
 	    } else {
-		/* "token/../foo" -> "foo" */
-		mhl_strmove (s, p + 4);
+		/* "foo/token/.." -> "foo" */
+		if (s == lpath + 1)
+		    s[0] = 0;
+		else
+		    s[-1] = 0;
+		break;
 	    }
-	    p = (s > lpath) ? s - 1 : s;
-	    continue;
-	}
 
-	/* trailing ".." */
-	if (s == lpath) {
-	    /* "token/.." -> "." */
-	    if (lpath[0] != PATH_SEP) {
-		lpath[0] = '.';
-	    }
-	    lpath[1] = 0;
-	} else {
-	    /* "foo/token/.." -> "foo" */
-	    if (s == lpath + 1)
-		s[0] = 0;
-	    else
-		s[-1] = 0;
 	    break;
 	}
-
-	break;
     }
 }
 
+void
+canonicalize_pathname (char *path)
+{
+    custom_canonicalize_pathname (path, CANON_PATH_ALL);
+}
+
 #ifdef HAVE_GET_PROCESS_STATS
 #    include <sys/procstats.h>
 
@@ -581,7 +648,7 @@ putenv (char *string)
     
     if (*ep == NULL){
 	static char **last_environ = NULL;
-	char **new_environ = g_new (char *, size + 2);
+	char **new_environ = g_try_new (char *, size + 2);
 	if (new_environ == NULL)
 	    return -1;
 	(void) memcpy ((void *) new_environ, (void *) __environ,
