@@ -1,5 +1,6 @@
 /* Virtual File System: GNU Tar file system.
-   Copyright (C) 2000 The Free Software Foundation
+   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2007
+   Free Software Foundation, Inc.
    
    Written by: 2000 Jan Hudec
 
@@ -17,11 +18,26 @@
    License along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
+/** \file
+ *  \brief Source: Virtual File System: GNU Tar file system.
+ *  \author Jan Hudec
+ *  \date 2000
+ */
+
 #include <config.h>
+
 #include <errno.h>
+#include <fcntl.h>
+
+#include "../src/global.h"
+
+#include "../src/wtools.h"	/* message() */
+#include "../src/main.h"	/* print_vfs_message */
+#include "../src/unixcompat.h"
+
 #include "utilvfs.h"
 #include "vfs-impl.h"
-#include "gc.h"		/* vfs_rmstamp */
+#include "gc.h"			/* vfs_rmstamp */
 #include "xdirentry.h"
 
 enum {
@@ -83,14 +99,14 @@ struct defer_inode {
     struct vfs_s_inode *inode;
 };
 
+/* FIXME: should be off_t instead of int. */
 static int cpio_position;
 
 static int cpio_find_head(struct vfs_class *me, struct vfs_s_super *super);
-static int cpio_read_bin_head(struct vfs_class *me, struct vfs_s_super *super);
-static int cpio_read_oldc_head(struct vfs_class *me, struct vfs_s_super *super);
-static int cpio_read_crc_head(struct vfs_class *me, struct vfs_s_super *super);
-static int cpio_create_entry(struct vfs_class *me, struct vfs_s_super *super, struct stat *, char *name);
-static int cpio_read(void *fh, char *buffer, int count);
+static ssize_t cpio_read_bin_head(struct vfs_class *me, struct vfs_s_super *super);
+static ssize_t cpio_read_oldc_head(struct vfs_class *me, struct vfs_s_super *super);
+static ssize_t cpio_read_crc_head(struct vfs_class *me, struct vfs_s_super *super);
+static ssize_t cpio_read(void *fh, char *buffer, int count);
 
 #define CPIO_POS(super) cpio_position
 /* If some time reentrancy should be needed change it to */
@@ -127,6 +143,9 @@ static int cpio_skip_padding(struct vfs_s_super *super)
 static void cpio_free_archive(struct vfs_class *me, struct vfs_s_super *super)
 {
     struct defer_inode *l, *lnext;
+
+    (void) me;
+
     if(super->u.arch.fd != -1)
 	mc_close(super->u.arch.fd);
 	super->u.arch.fd = -1;
@@ -146,7 +165,7 @@ cpio_open_cpio_file (struct vfs_class *me, struct vfs_s_super *super,
     struct vfs_s_inode *root;
 
     if ((fd = mc_open (name, O_RDONLY)) == -1) {
-	message (1, MSG_ERROR, _("Cannot open cpio archive\n%s"), name);
+	message (D_ERROR, MSG_ERROR, _("Cannot open cpio archive\n%s"), name);
 	return -1;
     }
 
@@ -155,14 +174,14 @@ cpio_open_cpio_file (struct vfs_class *me, struct vfs_s_super *super,
     mc_stat (name, &(super->u.arch.st));
     super->u.arch.type = CPIO_UNKNOWN;
 
-    type = get_compression_type (fd);
+    type = get_compression_type (fd, name);
     if (type != COMPRESSION_NONE) {
 	char *s;
 
 	mc_close (fd);
 	s = g_strconcat (name, decompress_extension (type), (char *) NULL);
 	if ((fd = mc_open (s, O_RDONLY)) == -1) {
-	    message (1, MSG_ERROR, _("Cannot open cpio archive\n%s"), s);
+	    message (D_ERROR, MSG_ERROR, _("Cannot open cpio archive\n%s"), s);
 	    g_free (s);
 	    return -1;
 	}
@@ -187,7 +206,7 @@ cpio_open_cpio_file (struct vfs_class *me, struct vfs_s_super *super,
     return fd;
 }
 
-static int cpio_read_head(struct vfs_class *me, struct vfs_s_super *super)
+static ssize_t cpio_read_head(struct vfs_class *me, struct vfs_s_super *super)
 {
     switch(cpio_find_head(me, super)) {
     case CPIO_UNKNOWN:
@@ -214,8 +233,8 @@ static int cpio_find_head(struct vfs_class *me, struct vfs_s_super *super)
 {
     char buf[256];
     int ptr = 0;
-    int top;
-    int tmp;
+    ssize_t top;
+    ssize_t tmp;
 
     top = mc_read (super->u.arch.fd, buf, 256);
     if (top > 0)
@@ -228,7 +247,7 @@ static int cpio_find_head(struct vfs_class *me, struct vfs_s_super *super)
 		top = 128;
 	    }
 	    if((tmp = mc_read(super->u.arch.fd, buf, top)) == 0 || tmp == -1) {
-		message (1, MSG_ERROR, _("Premature end of cpio archive\n%s"), super->name);
+		message (D_ERROR, MSG_ERROR, _("Premature end of cpio archive\n%s"), super->name);
 		cpio_free_archive(me, super);
 		return CPIO_UNKNOWN;
 	    }
@@ -251,14 +270,151 @@ static int cpio_find_head(struct vfs_class *me, struct vfs_s_super *super)
 #undef RETURN
 #undef SEEKBACK
 
+static int
+cpio_create_entry (struct vfs_class *me, struct vfs_s_super *super,
+		   struct stat *st, char *name)
+{
+    struct vfs_s_inode *inode = NULL;
+    struct vfs_s_inode *root = super->root;
+    struct vfs_s_entry *entry = NULL;
+    char *tn;
+
+    switch (st->st_mode & S_IFMT) {	/* For case of HP/UX archives */
+    case S_IFCHR:
+    case S_IFBLK:
+#ifdef S_IFSOCK
+    case S_IFSOCK:
+#endif
+#ifdef S_IFIFO
+    case S_IFIFO:
+#endif
+#ifdef S_IFNAM
+    case S_IFNAM:
+#endif
+	if ((st->st_size != 0) && (st->st_rdev == 0x0001)) {
+	    /* FIXME: representation of major/minor differs between */
+	    /* different operating systems. */
+	    st->st_rdev = (unsigned) st->st_size;
+	    st->st_size = 0;
+	}
+	break;
+    default:
+	break;
+    }
+
+    if ((st->st_nlink > 1)
+	     && ((super->u.arch.type == CPIO_NEWC)
+		    || (super->u.arch.type == CPIO_CRC))) {	/* For case of hardlinked files */
+	struct defer_inode i, *l;
+	i.inumber = st->st_ino;
+	i.device = st->st_dev;
+	i.inode = NULL;
+
+	l = cpio_defer_find (super->u.arch.deferred, &i);
+	if (l != NULL) {
+	    inode = l->inode;
+	    if (inode->st.st_size != 0 && st->st_size != 0
+		    && (inode->st.st_size != st->st_size)) {
+		message (D_ERROR, MSG_ERROR,
+			_("Inconsistent hardlinks of\n%s\nin cpio archive\n%s"),
+			name, super->name);
+		inode = NULL;
+	    } else if (inode->st.st_size == 0)
+		inode->st.st_size = st->st_size;
+	}
+    }
+
+    /* remove trailing slashes */
+    for (tn = name + strlen (name) - 1; tn >= name && *tn == PATH_SEP; tn--)
+	*tn = '\0';
+
+    tn = strrchr (name, PATH_SEP);
+    if (tn == NULL)
+	tn = name;
+    else if (tn == name + 1) {
+	/* started with "./" -- directory in the root of archive */
+	tn++;
+    } else {
+	*tn = '\0';
+	root = vfs_s_find_inode (me, super, name, LINK_FOLLOW, FL_MKDIR);
+	*tn = PATH_SEP;
+	tn++;
+    }
+
+    entry = MEDATA->find_entry (me, root, tn, LINK_FOLLOW, FL_NONE);	/* In case entry is already there */
+
+    if (entry != NULL) {
+	/* This shouldn't happen! (well, it can happen if there is a record for a
+	   file and than a record for a directory it is in; cpio would die with
+	   'No such file or directory' is such case) */
+
+	if (!S_ISDIR (entry->ino->st.st_mode)) {
+	    /* This can be considered archive inconsistency */
+	    message (D_ERROR, MSG_ERROR,
+		     _("%s contains duplicate entries! Skipping!"),
+		     super->name);
+	} else {
+	    entry->ino->st.st_mode = st->st_mode;
+	    entry->ino->st.st_uid = st->st_uid;
+	    entry->ino->st.st_gid = st->st_gid;
+	    entry->ino->st.st_atime = st->st_atime;
+	    entry->ino->st.st_mtime = st->st_mtime;
+	    entry->ino->st.st_ctime = st->st_ctime;
+	}
+
+	g_free (name);
+    } else {			/* !entry */
+	if (inode == NULL) {
+	    inode = vfs_s_new_inode (me, super, st);
+	    if ((st->st_nlink > 0)
+		     && ((super->u.arch.type == CPIO_NEWC)
+			    || (super->u.arch.type == CPIO_CRC))) {
+		/* For case of hardlinked files */
+		struct defer_inode *i;
+		i = g_new (struct defer_inode, 1);
+		i->inumber = st->st_ino;
+		i->device = st->st_dev;
+		i->inode = inode;
+		i->next = super->u.arch.deferred;
+		super->u.arch.deferred = i;
+	    }
+	}
+
+	if (st->st_size != 0)
+	    inode->data_offset = CPIO_POS (super);
+
+	entry = vfs_s_new_entry (me, tn, inode);
+	vfs_s_insert_entry (me, root, entry);
+
+	g_free (name);
+
+	if (!S_ISLNK (st->st_mode))
+	    CPIO_SEEK_CUR (super, st->st_size);
+	else {
+	    inode->linkname = g_malloc (st->st_size + 1);
+
+	    if (mc_read (super->u.arch.fd, inode->linkname, st->st_size) < st->st_size) {
+		inode->linkname[0] = '\0';
+		return STATUS_EOF;
+	    }
+
+	    inode->linkname[st->st_size] = '\0';	/* Linkname stored without terminating \0 !!! */
+	    CPIO_POS (super) += st->st_size;
+	    cpio_skip_padding (super);
+	}
+    }				/* !entry */
+
+    return STATUS_OK;
+}
+
 #define HEAD_LENGTH (26)
-static int cpio_read_bin_head(struct vfs_class *me, struct vfs_s_super *super)
+static ssize_t cpio_read_bin_head(struct vfs_class *me, struct vfs_s_super *super)
 {
     union {
 	struct old_cpio_header buf;
 	short shorts[HEAD_LENGTH >> 1];
     } u;
-    int len;
+    ssize_t len;
     char *name;
     struct stat st;
 
@@ -270,10 +426,10 @@ static int cpio_read_bin_head(struct vfs_class *me, struct vfs_s_super *super)
 	for(i = 0; i < (HEAD_LENGTH >> 1); i++)
 	    u.shorts[i] = GUINT16_SWAP_LE_BE_CONSTANT(u.shorts[i]);
     }
-    g_assert(u.buf.c_magic == 070707);
 
-    if (u.buf.c_namesize == 0 || u.buf.c_namesize > MC_MAXPATHLEN) {
-	message (1, MSG_ERROR, _("Corrupted cpio header encountered in\n%s"), 
+    if (u.buf.c_magic != 070707 ||
+	u.buf.c_namesize == 0 || u.buf.c_namesize > MC_MAXPATHLEN) {
+	message (D_ERROR, MSG_ERROR, _("Corrupted cpio header encountered in\n%s"), 
 		super->name);
 	return STATUS_FAIL;
     }
@@ -306,14 +462,14 @@ static int cpio_read_bin_head(struct vfs_class *me, struct vfs_s_super *super)
 #undef HEAD_LENGTH
 
 #define HEAD_LENGTH (76)
-static int cpio_read_oldc_head(struct vfs_class *me, struct vfs_s_super *super)
+static ssize_t cpio_read_oldc_head(struct vfs_class *me, struct vfs_s_super *super)
 {
     struct new_cpio_header hd;
     union {
 	struct stat st;
 	char buf[HEAD_LENGTH + 1];
     } u;
-    int len;
+    ssize_t len;
     char *name;
 
     if (mc_read (super->u.arch.fd, u.buf, HEAD_LENGTH) != HEAD_LENGTH)
@@ -322,16 +478,16 @@ static int cpio_read_oldc_head(struct vfs_class *me, struct vfs_s_super *super)
     u.buf[HEAD_LENGTH] = 0;
 
     if (sscanf (u.buf, "070707%6lo%6lo%6lo%6lo%6lo%6lo%6lo%11lo%6lo%11lo",
-	      &hd.c_dev, &hd.c_ino, &hd.c_mode, &hd.c_uid, &hd.c_gid,
-	      &hd.c_nlink, &hd.c_rdev, &hd.c_mtime,
+	      (unsigned long *)&hd.c_dev, &hd.c_ino, &hd.c_mode, &hd.c_uid, &hd.c_gid,
+	      &hd.c_nlink, (unsigned long *)&hd.c_rdev, &hd.c_mtime,
 	      &hd.c_namesize, &hd.c_filesize) < 10) {
-	message (1, MSG_ERROR, _("Corrupted cpio header encountered in\n%s"),
+	message (D_ERROR, MSG_ERROR, _("Corrupted cpio header encountered in\n%s"),
 		    super->name);
 	return STATUS_FAIL;
     }
 
     if (hd.c_namesize == 0 || hd.c_namesize > MC_MAXPATHLEN) {
-	message (1, MSG_ERROR, _("Corrupted cpio header encountered in\n%s"),
+	message (D_ERROR, MSG_ERROR, _("Corrupted cpio header encountered in\n%s"),
 		    super->name);
 	return STATUS_FAIL;
     }
@@ -365,27 +521,30 @@ static int cpio_read_oldc_head(struct vfs_class *me, struct vfs_s_super *super)
 #undef HEAD_LENGTH
 
 #define HEAD_LENGTH (110)
-static int cpio_read_crc_head(struct vfs_class *me, struct vfs_s_super *super)
+static ssize_t
+cpio_read_crc_head (struct vfs_class *me, struct vfs_s_super *super)
 {
     struct new_cpio_header hd;
     union {
 	struct stat st;
 	char buf[HEAD_LENGTH + 1];
     } u;
-    int len;
+    ssize_t len;
     char *name;
 
     if (mc_read (super->u.arch.fd, u.buf, HEAD_LENGTH) != HEAD_LENGTH)
 	return STATUS_EOF;
+
     CPIO_POS (super) += HEAD_LENGTH;
-    u.buf[HEAD_LENGTH] = 0;
+    u.buf[HEAD_LENGTH] = '\0';
 
     if (sscanf (u.buf, "%6ho%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx",
 	      &hd.c_magic, &hd.c_ino, &hd.c_mode, &hd.c_uid, &hd.c_gid,
 	      &hd.c_nlink,  &hd.c_mtime, &hd.c_filesize,
-	      &hd.c_dev, &hd.c_devmin, &hd.c_rdev, &hd.c_rdevmin,
+	      (unsigned long *)&hd.c_dev, (unsigned long *)&hd.c_devmin,
+	      (unsigned long *)&hd.c_rdev, (unsigned long *)&hd.c_rdevmin,
 	      &hd.c_namesize, &hd.c_chksum) < 14) {
-	message (1, MSG_ERROR, _("Corrupted cpio header encountered in\n%s"),
+	message (D_ERROR, MSG_ERROR, _("Corrupted cpio header encountered in\n%s"),
 		   super->name);
 	return STATUS_FAIL;
     }
@@ -395,14 +554,15 @@ static int cpio_read_crc_head(struct vfs_class *me, struct vfs_s_super *super)
 	return STATUS_FAIL;
 
     if (hd.c_namesize == 0 || hd.c_namesize > MC_MAXPATHLEN) {
-	message (1, MSG_ERROR, _("Corrupted cpio header encountered in\n%s"),
+	message (D_ERROR, MSG_ERROR, _("Corrupted cpio header encountered in\n%s"),
 		    super->name);
 	return STATUS_FAIL;
     }
 
     name = g_malloc(hd.c_namesize);
-    if((len = mc_read (super->u.arch.fd, name, hd.c_namesize)) == -1 ||
-       (unsigned long) len < hd.c_namesize) {
+    len = mc_read (super->u.arch.fd, name, hd.c_namesize);
+
+    if ((len == -1) || ((unsigned long) len < hd.c_namesize)) {
 	g_free (name);
 	return STATUS_EOF;
     }
@@ -410,144 +570,24 @@ static int cpio_read_crc_head(struct vfs_class *me, struct vfs_s_super *super)
     CPIO_POS(super) += len;
     cpio_skip_padding(super);
 
-    if(!strcmp("TRAILER!!!", name)) { /* We got to the last record */
+    if (strcmp ("TRAILER!!!", name) == 0) { /* We got to the last record */
 	g_free(name);
 	return STATUS_TRAIL;
     }
 
-    u.st.st_dev = (hd.c_dev << 8) + hd.c_devmin;
+    u.st.st_dev = makedev (hd.c_dev, hd.c_devmin);
     u.st.st_ino = hd.c_ino;
     u.st.st_mode = hd.c_mode;
     u.st.st_nlink = hd.c_nlink;
     u.st.st_uid = hd.c_uid;
     u.st.st_gid = hd.c_gid;
-    u.st.st_rdev = (hd.c_rdev << 8) + hd.c_rdevmin;
+    u.st.st_rdev = makedev (hd.c_rdev, hd.c_rdevmin);
     u.st.st_size = hd.c_filesize;
     u.st.st_atime = u.st.st_mtime = u.st.st_ctime = hd.c_mtime;
 
     return cpio_create_entry (me, super, &u.st, name);
 }
 
-static int
-cpio_create_entry (struct vfs_class *me, struct vfs_s_super *super,
-		   struct stat *st, char *name)
-{
-    struct vfs_s_inode *inode = NULL;
-    struct vfs_s_inode *root = super->root;
-    struct vfs_s_entry *entry = NULL;
-    char *tn;
-
-    switch (st->st_mode & S_IFMT) {	/* For case of HP/UX archives */
-    case S_IFCHR:
-    case S_IFBLK:
-#ifdef S_IFSOCK
-    case S_IFSOCK:
-#endif
-#ifdef S_IFIFO
-    case S_IFIFO:
-#endif
-#ifdef S_IFNAM
-    case S_IFNAM:
-#endif
-	if ((st->st_size != 0) && (st->st_rdev == 0x0001)) {
-	    st->st_rdev = (unsigned) st->st_size;
-	    st->st_size = 0;
-	}
-	break;
-    default:
-	break;
-    }
-
-    if ((st->st_nlink > 1) && (super->u.arch.type == CPIO_NEWC || super->u.arch.type == CPIO_CRC)) {	/* For case of hardlinked files */
-	struct defer_inode i, *l;
-	i.inumber = st->st_ino;
-	i.device = st->st_dev;
-	i.inode = NULL;
-	if ((l = cpio_defer_find (super->u.arch.deferred, &i)) != NULL) {
-	    inode = l->inode;
-	    if (inode->st.st_size && st->st_size
-		&& (inode->st.st_size != st->st_size)) {
-		message (1, MSG_ERROR,
-			 _
-			 ("Inconsistent hardlinks of\n%s\nin cpio archive\n%s"),
-			 name, super->name);
-		inode = NULL;
-	    } else if (!inode->st.st_size)
-		inode->st.st_size = st->st_size;
-	}
-    }
-
-    for (tn = name + strlen (name) - 1; tn >= name && *tn == PATH_SEP; tn--)
-	*tn = 0;
-    if ((tn = strrchr (name, PATH_SEP))) {
-	*tn = 0;
-	root = vfs_s_find_inode (me, super, name, LINK_FOLLOW, FL_MKDIR);
-	*tn = PATH_SEP;
-	tn++;
-    } else
-	tn = name;
-
-    entry = MEDATA->find_entry (me, root, tn, LINK_FOLLOW, FL_NONE);	/* In case entry is already there */
-
-    if (entry) {		/* This shouldn't happen! (well, it can happen if there is a record for a 
-				   file and than a record for a directory it is in; cpio would die with
-				   'No such file or directory' is such case) */
-
-	if (!S_ISDIR (entry->ino->st.st_mode)) {	/* This can be considered archive inconsistency */
-	    message (1, MSG_ERROR,
-		     _("%s contains duplicate entries! Skipping!"),
-		     super->name);
-	} else {
-	    entry->ino->st.st_mode = st->st_mode;
-	    entry->ino->st.st_uid = st->st_uid;
-	    entry->ino->st.st_gid = st->st_gid;
-	    entry->ino->st.st_atime = st->st_atime;
-	    entry->ino->st.st_mtime = st->st_mtime;
-	    entry->ino->st.st_ctime = st->st_ctime;
-	}
-
-    } else {			/* !entry */
-
-	if (!inode) {
-	    inode = vfs_s_new_inode (me, super, st);
-	    if ((st->st_nlink > 0) && (super->u.arch.type == CPIO_NEWC || super->u.arch.type == CPIO_CRC)) {	/* For case of hardlinked files */
-		struct defer_inode *i;
-		i = g_new (struct defer_inode, 1);
-		i->inumber = st->st_ino;
-		i->device = st->st_dev;
-		i->inode = inode;
-		i->next = super->u.arch.deferred;
-		super->u.arch.deferred = i;
-	    }
-	}
-
-	if (st->st_size)
-	    inode->data_offset = CPIO_POS (super);
-
-	entry = vfs_s_new_entry (me, tn, inode);
-	vfs_s_insert_entry (me, root, entry);
-
-	if (S_ISLNK (st->st_mode)) {
-	    inode->linkname = g_malloc (st->st_size + 1);
-	    if (mc_read (super->u.arch.fd, inode->linkname, st->st_size)
-		< st->st_size) {
-		inode->linkname[0] = 0;
-		g_free (name);
-		return STATUS_EOF;
-	    }
-	    inode->linkname[st->st_size] = 0;	/* Linkname stored without terminating \0 !!! */
-	    CPIO_POS (super) += st->st_size;
-	    cpio_skip_padding (super);
-	} else {
-	    CPIO_SEEK_CUR (super, st->st_size);
-	}
-
-    }				/* !entry */
-
-    g_free (name);
-    return STATUS_OK;
-}
-
 /* Need to CPIO_SEEK_CUR to skip the file at the end of add entry!!!! */
 
 static int
@@ -556,6 +596,8 @@ cpio_open_archive (struct vfs_class *me, struct vfs_s_super *super,
 {
     int status = STATUS_START;
 
+    (void) op;
+
     if (cpio_open_cpio_file (me, super, name) == -1)
 	return -1;
 
@@ -564,7 +606,7 @@ cpio_open_archive (struct vfs_class *me, struct vfs_s_super *super,
 
 	switch (status) {
 	case STATUS_EOF:
-	    message (1, MSG_ERROR, _("Unexpected end of file\n%s"), name);
+	    message (D_ERROR, MSG_ERROR, _("Unexpected end of file\n%s"), name);
 	    return 0;
 	case STATUS_OK:
 	    continue;
@@ -582,6 +624,10 @@ static void *
 cpio_super_check (struct vfs_class *me, const char *archive_name, char *op)
 {
     static struct stat sb;
+
+    (void) me;
+    (void) op;
+
     if (mc_stat (archive_name, &sb))
 	return NULL;
     return &sb;
@@ -593,6 +639,9 @@ cpio_super_same (struct vfs_class *me, struct vfs_s_super *parc,
 {
     struct stat *archive_stat = cookie;	/* stat of main archive */
 
+    (void) me;
+    (void) op;
+
     if (strcmp (parc->name, archive_name))
 	return 0;
 
@@ -608,7 +657,7 @@ cpio_super_same (struct vfs_class *me, struct vfs_s_super *parc,
     return 1;
 }
 
-static int cpio_read(void *fh, char *buffer, int count)
+static ssize_t cpio_read(void *fh, char *buffer, int count)
 {
     off_t begin = FH->ino->data_offset;
     int fd = FH_SUPER->u.arch.fd;
@@ -627,6 +676,9 @@ static int cpio_read(void *fh, char *buffer, int count)
 
 static int cpio_fh_open(struct vfs_class *me, struct vfs_s_fh *fh, int flags, int mode)
 {
+    (void) fh;
+    (void) mode;
+
     if ((flags & O_ACCMODE) != O_RDONLY) ERRNOR (EROFS, -1);
     return 0;
 }
