@@ -20,6 +20,17 @@
    License along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
+/**
+ * \file
+ * \brief Source: Virtual File System: garbage collection code
+ * \author Miguel de Icaza
+ * \author Jakub Jelinek
+ * \author Pavel Machek
+ * \author Pavel Roskin
+ * \date 1995, 1998, 2003
+ */
+
+
 #include <config.h>
 
 #include <stdio.h>
@@ -28,23 +39,21 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/types.h>
-#include <signal.h>
 #include <ctype.h>		/* is_digit() */
-
-#include <mhl/string.h>
+#include <sys/time.h>		/* gettimeofday() */
 
 #include "../src/global.h"
-#include "../src/tty.h"		/* enable/disable interrupt key */
+
 #include "../src/wtools.h"	/* message() */
 #include "../src/main.h"	/* print_vfs_message */
-#include "utilvfs.h"
-#include "vfs-impl.h"
-#include "gc.h"
-#include "vfs.h"
-
 #include "../src/panel.h"	/* get_current_panel() */
 #include "../src/layout.h"	/* get_current_type() */
 
+#include "utilvfs.h"
+#include "vfs-impl.h"
+#include "vfs.h"
+#include "gc.h"
+
 
 int vfs_timeout = 60;		/* VFS timeout in seconds */
 
@@ -124,7 +133,7 @@ vfs_getid (struct vfs_class *vclass, const char *dir)
     vfsid id = NULL;
 
     /* append slash if needed */
-    dir1 = mhl_str_dir_plus_file (dir, "");
+    dir1 = concat_dir_and_file (dir, "");
     if (vclass->getid)
 	id = (*vclass->getid) (vclass, dir1);
 
@@ -230,7 +239,7 @@ void
 vfs_expire (int now)
 {
     static int locked = 0;
-    struct timeval time;
+    struct timeval lc_time;
     struct vfs_stamping *stamp, *st;
 
     /* Avoid recursive invocation, e.g. when one of the free functions
@@ -239,11 +248,11 @@ vfs_expire (int now)
 	return;
     locked = 1;
 
-    gettimeofday (&time, NULL);
-    time.tv_sec -= vfs_timeout;
+    gettimeofday (&lc_time, NULL);
+    lc_time.tv_sec -= vfs_timeout;
 
     for (stamp = stamps; stamp != NULL;) {
-	if (now || (timeoutcmp (&stamp->time, &time))) {
+	if (now || (timeoutcmp (&stamp->time, &lc_time))) {
 	    st = stamp->next;
 	    if (stamp->v->free)
 		(*stamp->v->free) (stamp->id);
