@@ -1,21 +1,32 @@
+
+/**
+ * \file
+ * \brief Source: local FS
+ */
+
 #include <config.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <string.h>
+#include <fcntl.h>
 
+#include "../src/global.h"
+
+#include "../src/wtools.h"	/* message() */
+#include "../src/main.h"	/* print_vfs_message */
 
 #include "utilvfs.h"
-
 #include "vfs.h"
 #include "local.h"
 
-/* Note: Some of this functions are not static. This has rather good
+/**
+ * Note: Some of this functions are not static. This has rather good
  * reason: exactly same functions would have to appear in sfs.c. This
  * saves both computer's memory and my work.  <pavel@ucw.cz>
- * */
-    
+ */
+
 static struct vfs_class vfs_local_ops;
 
 static void *
@@ -24,6 +35,8 @@ local_open (struct vfs_class *me, const char *file, int flags, int mode)
     int *local_info;
     int fd;
 
+    (void) me;
+
     fd = open (file, NO_LINEAR (flags), mode);
     if (fd == -1)
 	return 0;
@@ -34,7 +47,7 @@ local_open (struct vfs_class *me, const char *file, int flags, int mode)
     return local_info;
 }
 
-int
+ssize_t
 local_read (void *data, char *buffer, int count)
 {
     int n;
@@ -70,6 +83,7 @@ local_close (void *data)
 int
 local_errno (struct vfs_class *me)
 {
+    (void) me;
     return errno;
 }
 
@@ -79,6 +93,8 @@ local_opendir (struct vfs_class *me, const char *dirname)
     DIR **local_info;
     DIR *dir;
 
+    (void) me;
+
     dir = opendir (dirname);
     if (!dir)
 	return 0;
@@ -108,12 +124,16 @@ local_closedir (void *data)
 static int
 local_stat (struct vfs_class *me, const char *path, struct stat *buf)
 {
+    (void) me;
+
     return stat (path, buf);
 }
 
 static int
 local_lstat (struct vfs_class *me, const char *path, struct stat *buf)
 {
+    (void) me;
+
 #ifndef HAVE_STATLSTAT
     return lstat (path,buf);
 #else
@@ -124,46 +144,59 @@ local_lstat (struct vfs_class *me, const char *path, struct stat *buf)
 int
 local_fstat (void *data, struct stat *buf)
 {
-    return fstat (*((int *) data), buf);    
+    /* FIXME: avoid type cast */
+    return fstat (*((int *) data), buf);
 }
 
 static int
 local_chmod (struct vfs_class *me, const char *path, int mode)
 {
+    (void) me;
+
     return chmod (path, mode);
 }
 
 static int
 local_chown (struct vfs_class *me, const char *path, int owner, int group)
 {
+    (void) me;
+
     return chown (path, owner, group);
 }
 
 static int
 local_utime (struct vfs_class *me, const char *path, struct utimbuf *times)
 {
+    (void) me;
+
     return utime (path, times);
 }
 
 static int
 local_readlink (struct vfs_class *me, const char *path, char *buf, size_t size)
 {
+    (void) me;
+
     return readlink (path, buf, size);
 }
 
 static int
 local_unlink (struct vfs_class *me, const char *path)
 {
+    (void) me;
+
     return unlink (path);
 }
 
 static int
 local_symlink (struct vfs_class *me, const char *n1, const char *n2)
 {
+    (void) me;
+
     return symlink (n1, n2);
 }
 
-static int
+static ssize_t
 local_write (void *data, const char *buf, int nbyte)
 {
     int fd;
@@ -188,16 +221,20 @@ local_write (void *data, const char *buf, int nbyte)
 static int
 local_rename (struct vfs_class *me, const char *a, const char *b)
 {
+    (void) me;
+
     return rename (a, b);
 }
 
 static int
 local_chdir (struct vfs_class *me, const char *path)
 {
+    (void) me;
+
     return chdir (path);
 }
 
-int
+off_t
 local_lseek (void *data, off_t offset, int whence)
 {
     int fd = * (int *) data;
@@ -208,30 +245,40 @@ local_lseek (void *data, off_t offset, int whence)
 static int
 local_mknod (struct vfs_class *me, const char *path, int mode, int dev)
 {
+    (void) me;
+
     return mknod (path, mode, dev);
 }
 
 static int
 local_link (struct vfs_class *me, const char *p1, const char *p2)
 {
+    (void) me;
+
     return link (p1, p2);
 }
 
 static int
 local_mkdir (struct vfs_class *me, const char *path, mode_t mode)
 {
+    (void) me;
+
     return mkdir (path, mode);
 }
 
 static int
 local_rmdir (struct vfs_class *me, const char *path)
 {
+    (void) me;
+
     return rmdir (path);
 }
 
 static char *
 local_getlocalcopy (struct vfs_class *me, const char *path)
 {
+    (void) me;
+
     return g_strdup (path);
 }
 
@@ -239,28 +286,20 @@ static int
 local_ungetlocalcopy (struct vfs_class *me, const char *path,
 		      const char *local, int has_changed)
 {
+    (void) me;
+    (void) path;
+    (void) local;
+    (void) has_changed;
+
     return 0;
 }
 
-#ifdef HAVE_MMAP
-caddr_t
-local_mmap (struct vfs_class *me, caddr_t addr, size_t len, int prot, int flags, void *data, off_t offset)
-{
-    int fd = * (int *)data;
-
-    return mmap (addr, len, prot, flags, fd, offset);
-}
-
-int
-local_munmap (struct vfs_class *me, caddr_t addr, size_t len, void *data)
-{
-    return munmap (addr, len);
-}
-#endif
-
 static int
 local_which (struct vfs_class *me, const char *path)
 {
+    (void) me;
+    (void) path;
+
     return 0;		/* Every path which other systems do not like is expected to be ours */
 }
 
@@ -296,9 +335,5 @@ init_localfs (void)
     vfs_local_ops.ungetlocalcopy = local_ungetlocalcopy;
     vfs_local_ops.mkdir = local_mkdir;
     vfs_local_ops.rmdir = local_rmdir;
-#ifdef HAVE_MMAP
-    vfs_local_ops.mmap = local_mmap;
-    vfs_local_ops.munmap = local_munmap;
-#endif
     vfs_register_class (&vfs_local_ops);
 }
