@@ -21,11 +21,19 @@
    License along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
-/* Namespace: exports mcfs_vfs_ops, tcp_invalidate_socket */
+/**
+ * \file
+ * \brief Source: Midnight Commander file system
+ * \author Miguel de Icaza
+ * \author Andrej Borsenkow
+ * \author Norbert Warmuth
+ *
+ * Namespace: exports mcfs_vfs_ops, tcp_invalidate_socket
+ */
 
 #include <config.h>
 
-#ifdef WITH_MCFS
+#ifdef ENABLE_VFS_MCFS
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
@@ -50,7 +58,6 @@
 #endif
 
 #include "../src/global.h"
-#include "../src/tty.h"		/* enable/disable interrupt key */
 #include "../src/wtools.h"	/* message() */
 #include "../src/main.h"	/* print_vfs_message */
 #include "utilvfs.h"
@@ -130,7 +137,7 @@ mcfs_login_server (int my_socket, char *user, int port,
 	return 0;
 
     if (result != MC_VERSION_OK) {
-	message (1, _(" MCFS "),
+	message (D_ERROR, _(" MCFS "),
 		    _(" The server does not support this version "));
 	close (my_socket);
 	return 0;
@@ -151,7 +158,7 @@ mcfs_login_server (int my_socket, char *user, int port,
 			      (" The remote server is not running on a system port \n"
 			       " you need a password to log in, but the information may \n"
 			       " not be safe on the remote side.  Continue? \n"),
-			      3, 2, _("&Yes"), _("&No"));
+			      D_ERROR, 2, _("&Yes"), _("&No"));
 
 	    if (v == 1) {
 		close (my_socket);
@@ -175,7 +182,7 @@ mcfs_login_server (int my_socket, char *user, int port,
 	    return 0;
 
 	if (result != MC_LOGINOK) {
-	    message (1, _(" MCFS "), _(" Invalid password "));
+	    message (D_ERROR, _(" MCFS "), _(" Invalid password "));
 	    rpc_send (my_socket, RPC_INT, MC_QUIT, RPC_END);
 	    close (my_socket);
 	    return 0;
@@ -235,7 +242,7 @@ mcfs_create_tcp_link (const char *host, int *port, int *version, const char *cal
 		sizeof (inaddr));
     else {
 	if ((hp = gethostbyname (host)) == NULL) {
-	    message (1, caller, _(" Cannot locate hostname: %s "),
+	    message (D_ERROR, caller, _(" Cannot locate hostname: %s "),
 			host);
 	    return 0;
 	}
@@ -254,13 +261,13 @@ mcfs_create_tcp_link (const char *host, int *port, int *version, const char *cal
     server_address.sin_port = htons (*port);
 
     if ((my_socket = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
-	message (1, caller, _(" Cannot create socket: %s "),
+	message (D_ERROR, caller, _(" Cannot create socket: %s "),
 		    unix_error_string (errno));
 	return 0;
     }
     if (connect (my_socket, (struct sockaddr *) &server_address,
 		 sizeof (server_address)) < 0) {
-	message (1, caller, _(" Cannot connect to server: %s "),
+	message (D_ERROR, caller, _(" Cannot connect to server: %s "),
 		    unix_error_string (errno));
 	close (my_socket);
 	return 0;
@@ -330,7 +337,7 @@ mcfs_open_link (char *host, char *user, int *port, char *netrcpass)
 		return &mcfs_connections[i];
 	}
     if (mcfs_open_connections == MCFS_MAX_CONNECTIONS) {
-	message (1, MSG_ERROR, _(" Too many open connections "));
+	message (D_ERROR, MSG_ERROR, _(" Too many open connections "));
 	return 0;
     }
 
@@ -406,7 +413,7 @@ mcfs_get_path (mcfs_connection **mc, const char *path)
 	int f = !strcmp (remote_path, "/~");
 	if (f || !strncmp (remote_path, "/~/", 3)) {
 	    char *s;
-	    s = mhl_str_dir_plus_file (mcfs_gethome (*mc),
+	    s = concat_dir_and_file (mcfs_gethome (*mc),
 				     remote_path + 3 - f);
 	    g_free (remote_path);
 	    remote_path = s;
@@ -551,7 +558,7 @@ mcfs_open (struct vfs_class *me, const char *file, int flags, int mode)
     return remote_handle;
 }
 
-static int
+static ssize_t
 mcfs_read (void *data, char *buffer, int count)
 {
     mcfs_handle *info = (mcfs_handle *) data;
@@ -578,7 +585,7 @@ mcfs_read (void *data, char *buffer, int count)
     return result;
 }
 
-static int
+static ssize_t
 mcfs_write (void *data, const char *buf, int nbyte)
 {
     mcfs_handle *info = (mcfs_handle *) data;
@@ -686,17 +693,17 @@ mcfs_loaddir (opendir_info *mcfs_info)
 {
     int status, error;
     mcfs_connection *mc = mcfs_info->conn;
-    int link = mc->sock;
+    int lc_link = mc->sock;
     int first = 1;
 
-    rpc_send (link, RPC_INT, MC_READDIR, RPC_INT, mcfs_info->handle,
+    rpc_send (lc_link, RPC_INT, MC_READDIR, RPC_INT, mcfs_info->handle,
 	      RPC_END);
 
     for (;;) {
 	int entry_len;
 	dir_entry *new_entry;
 
-	if (!rpc_get (link, RPC_INT, &entry_len, RPC_END))
+	if (!rpc_get (lc_link, RPC_INT, &entry_len, RPC_END))
 	    return 0;
 
 	if (entry_len == 0)
@@ -716,11 +723,11 @@ mcfs_loaddir (opendir_info *mcfs_info)
 	}
 
 	if (!rpc_get
-	    (link, RPC_BLOCK, entry_len, new_entry->text, RPC_END))
+	    (lc_link, RPC_BLOCK, entry_len, new_entry->text, RPC_END))
 	    return 0;
 
 	/* Then we get the status from the lstat */
-	if (!rpc_get (link, RPC_INT, &status, RPC_INT, &error, RPC_END))
+	if (!rpc_get (lc_link, RPC_INT, &status, RPC_INT, &error, RPC_END))
 	    return 0;
 
 	if (mcfs_is_error (status, error))
@@ -1069,7 +1076,7 @@ mcfs_chdir (struct vfs_class *me, const char *path)
     return 0;
 }
 
-static int
+static off_t
 mcfs_lseek (void *data, off_t offset, int whence)
 {
     mcfs_handle *info = (mcfs_handle *) data;
@@ -1246,5 +1253,4 @@ tcp_invalidate_socket (int sock)
 {
     mcfs_invalidate_socket (sock);
 }
-
-#endif				/* WITH_MCFS */
+#endif				/* ENABLE_VFS_MCFS */
